;+
; Name: fermi_get_gti
;
; Purpose:  Get times of valid Fermi data (not in SAA).  This is available in the GTI extension of the
;   daily cspec pha files.
;
; Calling sequence: times = fermi_get_gti(date)
;
; Input:
;   date - either a single time (assumes entire day), or a time range, in anytim
;          format (if seconds then sec since 1979/1/1).  Will do whole days.
;          
; Input Keywords:
;   cleanup - if set, remove any files copied.  Default is 1.
;   
; Output Keywords:
;   status - 0 / 1 means fail /success
;   
; Examples:
;   ptim, fermi_get_dn('12-dec-2015', cleanup=0)
;   ptim, fermi_get_dn(['12-dec-2015', '15-dec-2015 23:00'], status=status)
;   
; Written: Kim Tolbert, 25-Feb-2016
;
;-

function fermi_get_dn, date, cleanup=cleanup, status=status

checkvar, cleanup, 1

status = 0

time_range = n_elements(date) eq 1 ? anytim(date, /date_only) + [0,86400.d0] : anytim(date)

gbm_find_data, date=time_range, pattern='_poshist_',count=count, file=file
if count eq 0 then begin
  message, /cont, 'No CSPEC PHA files found for times requested. Aborting.'
  return, -1
endif

for ifile=0,count-1 do begin
  dn = fermi_dn(file[ifile])
  find_changes, dn.mask, dni, dns, count=dnc, index2d=dni2
  times = anytim(dn[dni2].ut)
  mask = dn[dni2].mask
  q = where(mask[0,*] eq 1)
  times = times[*,q]
  
  if ifile eq 0 then begin
    daytimes = times
  endif else begin
    ; After first file, merge last interval already saved with first new one, if they overlap
    if times[0,0] lt last_item(daytimes) then begin
      daytimes[n_elements(daytimes)-1] = times[1,0]
      times = times[*,1:*]
    endif
    daytimes = [[daytimes],[times]]
  endelse
  
endfor

if cleanup then file_delete, file, /quiet

status = 1
return, daytimes
end
;ptim,times[*,q]