;+
; Project: SPEX
;
; Purpose:  Function to return information about a function (which can consist of multiple components, e.g. vth+bpow).
;
; Input arguments:
;  fit_function - fit_function string to return info for.  list of components or component strategies, separated by '+',
;    e.g. thick2_vnorm+line+pileup_mod, or thick2_vnorm#0+line#0+line#1+pilup_mod#0
;    When you want to return info for a specific component of which there's more than one, use the 'component with strategy' version of fit_function,
;    and include the strategy in the comp keyword (e.g. 'line#1') or use the comp keyword = number of the component.
;
; Input Keywords:
;   nparams - if set, return the total number of parameters
;   ncomp - if set, return the total number of function components
;   defaults - if set, return a structure with default values
;   comp - only used with param_index or param_elem. Can be one of the following:
;      Name of the component - e.g. 'line' (if func has more than one line, will use first one)
;      Number of the component (from 0), e.g. if func='vth+bpow+line+line', comp=3 means 2nd line
;      Name#number - e.g. line#1.  If fit_function passed in contains the strategy number, we find the name#number in the fit_function list.
;      If fit_function passed in does not contain the strategy number, line#0 means the first line component, line#1 means the second;   
;   param_index - if set, return arrays of start/end values of parameters for each component in form (nfunc,2).
;     For example - a = fit_function_query('vth+bpow', /param_index) returns
;      a[0,*] = [0,2]  - parameters for 'vth'
;      a[1,*] = [3,6]  - parameters for 'bpow'
;     If comp keyword is set, return indices into parameters for just that component
;      For example, a = fit_function_query('vth+bpow', /param_index, comp='bpow') returns
;      a = [2,5] meaning parameters for 'bpow' are in elements 2 through 5 of param array
;      Example 3: fit_function_query('vth#0+line#1+line#0', comp='line#1', /param_index) returns
;      a = [3,5] meaning parameters for line#1 component are in elements 3 through 5 of param array
; param_elem - if set, return the array of element numbers (not start/end indices like param_index returns)
;    Useful when comp is passed (otherwise it's just an indgen of the number of parameters). For example,
;    IDL> print,fit_function_query('vth+bpow', comp='vth', /param_elem)
;         0           1           2
;    IDL> fit_function_query('vth#0+line#1+line#0', comp='line#1', /param_elem)
;         3           4           5     
; param_desc - if set, return an array of parameter descriptions for all parameters for all components
; param_num - if set, returns each comp name followed by P0, P1, etc for each parameter, for all comps
;   e.g. a=fit_function_query('vth+bpow',/param_num)
;        help,a
;          A               STRING    = Array[7]
;       print,arr2str(a,', ')
;       vth P0, vth P1, vth P2, bpow P0, bpow P1, bpow P2, bpow P3
;  add_strat - if set, modify fit function to add strategy numbers, i.e. 'vth+bpow+line+line' becomes 'vth#0+bpow#0+line#0+line#1'
;
; ...
; Modifications:
;  20-Mar-2006, Kim.  Added ncomp keyword
;  4-Feb-2008, Kim.  Comp can now include information about which comp we want if there
;    are multiple instances of comp.  comp='line#2' means the 3rd instance of line
;  13-Sep-2012, Kim. Added param_desc and param_num keywords
;  26-Aug-2013, Kim. Added param_elem keyword
;  14-nov-2013, Kim. Minor doc changes
;  23-Mar-2023, Kim.  Did not always work correctly when requesting a component in a function that had more than one of those components.  Fixed.
;    Made it possible to pass in fit_function as an array of comp with strategy number to specify which comp we want. Added add_strat keyword
;    so that if fit_function doesn't have strategy numbers, we append them (defaulting to putting them in order).  So vth+bpow will become
;    vth#0+bpow#0. Strat numbers can be out of order if e.g. a user deleted the first instance of a component. i.e. vth#0+line#0+line#1 becomes
;    vth#0+line#1 if user deletes the first line component.  
;-

function fit_function_query, fit_function, nparams=nparams, ncomp=ncomp, $
  defaults=defaults, param_index=param_index, comp=comp, param_elem=param_elem, $
  param_desc=param_desc, param_num=param_num, add_strat=add_strat

  ;Note: the add_strat block must be first, otherwise get into infinite loop, since calls this below.
  if keyword_set(add_strat) then begin
    ; assume the strategy # is in order, since don't know otherwise, e.g. 2nd line component is line#1
    ; (since they can be out of order)

    ;if already have a # in fit_function, assume already has strategy numbers
    if strpos(fit_function, '#') ne -1 then return, fit_function
    
    fit_comp_arr = str2arr(fit_function, '+')
    c_uniq = get_uniq(fit_comp_arr, count=nuniq)
    new = fit_comp_arr
    for i=0,nuniq-1 do begin
      q = where(c_uniq[i] eq fit_comp_arr, count)
      new[q] = c_uniq[i] + '#' + trim(indgen(count))
    endfor
    return, arr2str(new, '+')
  endif

  if strpos(fit_function, '#') eq -1 then begin
    fit_func_nostrat = fit_function    
    fit_func_strat = fit_function_query(fit_function, /add_strat)
    fit_comp_arr = str2arr(fit_function, '+')
    fit_strat_arr = str2arr(fit_func_strat, '+')
  endif else begin
    fit_strat_arr = str2arr(fit_function, '+')
    fit_comp_arr = ssw_strsplit(fit_strat_arr, '#')
    fit_func_nostrat = arr2str(fit_comp_arr, '+')
    fit_func_strat = fit_function
  endelse

  nc = n_elements(fit_comp_arr)

  if keyword_set(ncomp) then return, nc

  num_params = 0
  for i=0,nc-1 do num_params = num_params + fit_comp_defaults(fit_comp_arr[i], /nparam)

  if keyword_set(nparams) then return, num_params

  if keyword_set(defaults) then begin
    if nc eq 1 then return, fit_comp_defaults(fit_func_nostrat) else $
      ; if more than one component, let fit_function object concat values from separate components
      return, fit_function_defaults(fit_func_nostrat)
  endif

  if keyword_set(param_index) or keyword_set(param_elem) then begin
    if exist(comp) then begin
      if is_string(comp) then begin ; component requested is in keyword comp (if doesn't have a strat, append '#0')
      comp2 = strpos(comp,'#') eq -1 ? comp + '#0' : comp
      comp_ind = where(fit_strat_arr eq comp2, count)
      if count eq 0 then return, -1
      endif else begin ; element number of component requested was entered in comp keyword
        comp_ind = comp
        if comp_ind gt nc-1 then return, -1
      endelse
    endif

    ; Get all parameter indices
    ind = 0
    for i = 0,nc-1 do begin
      nparam = fit_comp_defaults(fit_comp_arr[i], /nparam)
      i1 = append_arr(i1, ind)
      i2 = append_arr(i2, ind+nparam-1)
      ind = ind + nparam
    endfor
    
    ; if comp specified, return indices for just that component
    if exist(comp) then begin
      comp_ind = comp_ind[0]
      is = i1[comp_ind] & ie = i2[comp_ind]
      return, keyword_set(param_elem) ? is + indgen(ie-is+1): [is,ie]
    endif else return, keyword_set(param_elem) ? indgen(num_params) : [[i1],[i2]]
  endif

  if keyword_set(param_desc) then begin
    for i=0,nc-1 do begin
      fit_struct = fit_model_components(fit_comp_arr[i], /struct)
      desc = append_arr(desc, fit_comp_arr[i] + ': ' + fit_struct.param_desc)
    endfor
    return, desc
  endif

  if keyword_set(param_num) then begin
    for i=0,nc-1 do begin
      nparam = fit_comp_defaults(fit_comp_arr[i], /nparam)
      pnum = append_arr(pnum, fit_comp_arr[i] + ' P' + trim(indgen(nparam)))
    endfor
    return, pnum
  endif

  return,0

end