;+
;
; NAME:	rd_grs_fits
;
; PURPOSE:	Read smm grs fits files into idl. Returns either PHA data or HEM data as requested. For HEM data, 18 channels are not
;   really energy bins - define fake energy edges of 1 keV bins starting at 1, also area is set to 1, and livetime is set to duration of interval.
;
; CATEGORY:	grs
;
; CALLING SEQUENCE:
;       rd_grs_fits, filename, ut, counts, livetime, edges, deltat
;
; CALLS: mrdfits
;
; INPUTS:
;  Filename:	grs fits file name
;
; INPUT KEYWORDS:
;  hem - if set, return GRS HEM (High Energy Matrix) data instead of PHA data
;
; OUTPUTS:
;  Ut:		Time in seconds since 1-jan-1979, middle of datapoint, dblarr(nbin)
;	 Counts:		Number of counts in each of energy/time bin, fltarr(nen,nbin)
;	 Livetime:	Livetime in sec in each sample, fltarr(nen,nbin)
;	 Edges:		Output energy loss bands in keV, fltarr(2,nen)
;	 Deltat:		Time bin width in sec, fltarr(nbin)
;	 Ercounts: Error in Counts fltarr(nen,nbin)
;
; OPTIONAL OUTPUTS:
;  H0:		Primary header
;	 H1:	 	Header from first extension
;	 H2:		Header from second (energy edges) extension
;	 Error:		Returns 1 if problem.
;
; Written: Kim Tolbert, 19-Mar-2019, based on rd_hxrbs_fits
; MODIFICATION:
; 22-Apr-2019, Kim.  Added returning ercounts.  In usual case, ercounts is the sqrt(Counts),
;  and previously just assumed that in calling routine, but now we also have files that have
;  the background-subtracted counts, and error is not just sqrt(Counts). So return the stat_err
;  field from the data structure converted to counts.
; 12-Jun-2019, Kim. Added hem option to return High Energy Matrix data (from  3rd extension)
;-

pro rd_grs_fits, filename, hem=hem, ut, counts, ercounts, livetime, edges, deltat, error=error,$
  h0=h0, h1=h1, h2=h2

  checkvar, hem, 0

  error = 0
  filename= (findfile(filename[0],count=nfile))[0]
  if nfile ne 1 then begin
    print,'Error reading ',filename
    error=1
    return
  endif

  dummy = mrdfits(filename, 0, h0, /silent)
  data = mrdfits(filename, 1, h1, /silent)

  ut = data.time
  deltat = data.timedel

  if hem then begin
    ; HEM data uses same time bins as pha data so already have those
    ; Read 3rd extension to get structure with hem data
    str = mrdfits(filename, 3, h3, /silent)
    ; a.hem will be (18,8,ntime), 18 are the counts in the different interactions in NaI,CsI detector layers, 8 are the 8 time bins within
    ; a major frame, and ntime are the major frames. We'll total over the 8 bins within the major frame to have just mf boundary data which
    ; will then be on same time intervals as main pha data.
    counts = total(str.hem,2)  ; total over the 8 sub-mf time bins

    ercounts = sqrt(counts)

    ;    ; --- This calculation of HEM energy bins is extracted directly from grs_energy.pro and grs_lightcurve.pro (case branch for hem=1)
    ;    ; except renamed hem to hem_e so doesn't interfere with input keyword
    ;    HEM_SCALE = [0.0,1.0,2.5,4.0,6.5,10.0] ;scale by 10.2 MeV for NaI, 12.7 for CsI
    ;    H04 = [[0,1],[4,5]]*1.
    ;    H03 = [[0,1],[3,4]]*1.
    ;    H13 = [[1,2],[3,4]]*1.
    ;    H23 = [[2,3],[3,4]]*1.
    ;    H02 = [[0,1],[2,3]]*1.
    ;    H12 = [[1,2],[2,3]]*1.
    ;    H22 = [[2,3],[2,3]]*1.
    ;    H32 = [[3,4],[2,3]]*1.
    ;    H42 = [[4,5],[2,3]]*1.
    ;    H01 = [[0,1],[1,2]]*1.    ;less than 25 MeV
    ;    H11 = [[1,2],[1,2]]*1.
    ;    H21 = [[2,3],[1,2]]*1.
    ;    H31 = [[3,4],[1,2]]*1.
    ;    H41 = [[4,5],[1,2]]*1.
    ;    H10 = [[1,2],[0,1]]*1.    ;less than 25 MeV
    ;    H20 = [[2,3],[0,1]]*1.
    ;    H30 = [[3,4],[0,1]]*1.
    ;    H40 = [[4,5],[0,1]]*1.
    ;
    ;    HEM_E = {energy_hem, units: 'MeV', H04:H04,H03:H03,H13:H13,H23:H23,H02:H02,H12:H12,H22:H22,$
    ;      H32:H32, H42:H42,H01:H01,H11:H11,H21:H21,H31:H31,H41:H41,H10:H10, $
    ;      H20:H20, H30:H30,H40:H40}
    ;    for i=1,18 do HEM_E.(I) = $
    ;      [[10.2*HEM_SCALE(HEM_E.(I)(*,0))],[12.7*HEM_SCALE(HEM_E.(I)(*,1))]]
    ;    e = {hem:hem_e}
    ;    energy=fltarr(2,2,18)
    ;    for i=0,17 do energy(0,0,i)=e.hem.(1+i)
    ;    energy = total(energy,2)
    ;
    ;    ; -- end of extracted code
    ;
    ;    edges = energy * 1.e3 ; convert to keV

    ; Those energies are not ordered and overlap.  We'll assign fake 1 keV bins instead.
    edges = get_edges(findgen(19) + 1., /edges_2)
    livetime = (fltarr(18)+1.) # data.timedel   ; no livetime measurement for HEM, just use duration of interval

  endif else begin

    ; Main PHA data
    counts = data.rate * data.livetime
    ercounts = data.stat_err * data.livetime
    eneband = mrdfits(filename, 2, h2, /silent)
    edges = transpose([[eneband.e_min], [eneband.e_max]])
    livetime = data.livetime

  endelse

end
