;+ This collection of routines reads the files 'ALLFLARES.DAT' and 'ALLFLARES.LIST' provided by G. Share
;  which contain bk-subtracted, energy grouped count rates for 185 SMM/GRS events. The information is merged
;  into an array of structures called groups, which is written to a sav file. That sav file is used as input 
;  to the routine grs_bksub_egrouped_2fits, which writes a FITS file for each group (event).
;-
;  
;+
; :Description:
;    Returns a structure with the index range of the channel groups based
;    on the spectrum bin centers and widths obtained from the group structure input
; :Returns:
;    E.G.
;    IDL> help, *groups[0].igroup,/st
;    ** Structure <213c3ce0>, 4 tags, length=192, data length=192, refs=1:
;    IGROUP          LONG      Array[2, 8]
;    MCHAN           FLOAT     Array[2, 8]
;    DCENTROID       FLOAT     Array[8]
;    DWIDTH          FLOAT     Array[8]
;
; :Params:
;    group - input data structure with pointer spec that contains the grouped channel
;    information from the allflares.dat file provided by G Share in March 2019
;
; :Keywords:
;    echan - data bins in keV for SMM/GRS instrument
;    path  - path to folder with gerry_grs_drmnew.sav file. Not needed if echan provided
;
; :Author: raschwar, 28-mar-2019
;-
function grs_alldat_group, group, echan = echan, path = path

  on_error, 2
  if ~same_data( size(/dim, echan), [2L, 476L]) then begin
    default, path, 'c:\ssw\soft\smmgrs'

    grs_drmfile= loc_file( path=path, 'gerry_grs_drmnew.sav', count=gcount)
    eout = 0
    ein = 0
    if gcount eq 0 then message,'No grs drmfile' else restore, grs_drmfile[0]

    echan = eout
  endif
  edge_products, echan, mean = emc, width = wc
  mechan = echan / 1.0e3
  ; Description of the spec array from *group.spec, From Gerry Share
  ; Attached is an ascii file providing spectrally integrated counts spectra of the SMM flares where I have summed up data channels
  ; to make sure that the background-corrected count rate in each channel is significant at  > two sigma.
  ; Column one gives the center energy (MeV) and two gives a rough estimate of the width, three gives the count rate (cts/MeV-s),
  ; and four gives the error.  The fifth column is a best fit rate done back in 1994.

  spec = *group.spec
  nchan = n_elements( spec[0,*] )
  elo  = spec[0,*]-spec[1,*]/2
  ehi  = spec[0,*]+spec[1,*]/2
  ilo  = value_closest( reform( mechan[0,*]), elo )
  ihi  = value_closest( reform( mechan[1,*]), ehi )
  igroup = [ ilo,ihi]
  egchan = [ mechan[0,igroup[0,*]], mechan[1,igroup[1,*]] ]* 1000.
  mk_contiguous, egchan, cedges, test=test
  if ~test then stop
  out = { igroup: igroup, mchan: egchan/1000., dcentroid: reform( spec[0,*]  ) - avg( egchan/1000.,0), $
    dwidth: reform( spec[1,*] ) - get_edges(/width, egchan/1000.) }

  return, out
end



;+
; :Description:
;    Reads and parses 'ALLFLARES.LIST'
;    Returns a structure with the event start and stop times and livetime
;
; :Examples:
; IDL> help, grs_allflares_read(),/st
;      ** Structure <210fb0d0>, 3 tags, length=24, data length=20, refs=1:
;      TSTART          DOUBLE           39172677.
;      TEND            DOUBLE           39172709.
;      LIVETIME        FLOAT           29.8980

;
; :Keywords:
;    path- path to folder containing 'ALLFLARES.LIST'
;     defaults to $sswdb_smm
;
; :Author: raschwar, 28-mar-2019
;-
function grs_allflares_read, path = path

  default, path, '$sswdb_smm'
  default, file,'ALLFLARES.LIST'
  dfile = loc_file( path = path, file )
  a = rd_ascii( dfile )
  tline = where( stregex( a, /fold, /boo, '(DIFSPE D)'), ntline)
  title = a[tline]
  livetime = float( strmid( title, 76, 8))

  year  = fix( strmid( title, 20,4 ))
  doy   = fix( strmid( title, 25,4 ))
  time  = strmid( title, 29, 12)
  tstart = dblarr( ntline )
  for i = 0, ntline -1 do tstart[i] = anytim( doytodate( year[i], doy[i], /yoh, /date) + ' '+ time[i]) - 8.192
  tmjf   = a[tline+1]
  p      = strpos( tmjf, 'MJFP')

  nmjf   = doy * 0
  for i = 0, ntline -1 do nmjf[i] = fix( strmid( tmjf[i],0, p[i]) )
  tend   = tstart + nmjf * 16.384
  out = replicate( { tstart: 0.d0, tend:0.d0, livetime: 0.0}, ntline)
  out.tstart = tstart
  out.tend = tend
  out.livetime = livetime
  return, out
end

;+
; :Description:
; Two files, and 'ALLFLARES.DAT' and 'ALLFLARES.LIST' are processed to create a set of energy compressed background subtraced
; data for 185 SMM/GRS flares.  
; 'ALLFLARES.DAT' is an ascii file providing spectrally integrated counts spectra of the SMM flares where I have summed up data channels
; to make sure that the background-corrected count rate in each channel is significant at  > two sigma.
; Column one gives the center energy (MeV) and two gives a rough estimate of the width, three gives the count rate (cts/MeV-s),
; and four gives the error.  The fifth column is a best fit rate done back in 1994.
; 
; 
; For the first flare in ALLFLARES.DAT
; 1980-MAR-29
;    8
;    0.302       0.014     157.622      59.391     210.947
;    0.316       0.014     160.568      51.655     177.148
;    0.330       0.014     216.437      47.546     147.076
;    0.348       0.022      99.485      34.029     118.439
;    0.369       0.022     133.999      35.403      90.769
;    0.403       0.045      59.549      23.489      63.445
;    0.495       0.138      28.504      12.848      28.059
;    4.805       8.482       0.200       0.425       0.247
;    
;
; :Examples:
;
;      IDL> grs_spectra = grs_alldat_process()
;      IDL> help, grs_spectra
;      GRS_SPECTRA     STRUCT    = -> <Anonymous> Array[185]
;      IDL> help, grs_spectra,/st
;      ** Structure <1e4b7d80>, 9 tags, length=64, data length=54, refs=1:
;      TSTART          DOUBLE           39172677.
;      TEND            DOUBLE           39172709.
;      LIVETIME        FLOAT           29.8980
;      DATE            STRING    '1980-MAR-29'
;      NCHAN           INT              8
;      SPEC            POINTER   <PtrHeapVar2969>
;      IGROUP          POINTER   <PtrHeapVar2971>
;      GRP_DRM         POINTER   <PtrHeapVar2972>
;      GRP_EOUT        POINTER   <PtrHeapVar2973>
;      IDL> print,*(grs_spectra[0]).spec
;      0.302000    0.0140000      157.622      59.3910      210.947
;      0.316000    0.0140000      160.568      51.6550      177.148
;      0.330000    0.0140000      216.437      47.5460      147.076
;      0.348000    0.0220000      99.4850      34.0290      118.439
;      0.369000    0.0220000      133.999      35.4030      90.7690
;      0.403000    0.0450000      59.5490      23.4890      63.4450
;      0.495000     0.138000      28.5040      12.8480      28.0590
;      4.80500      8.48200     0.200000     0.425000     0.247000
;      IDL> print,*(grs_spectra[0]).grp_eout
;            294.941      308.747
;            308.747      322.744
;            322.744      336.932
;            336.932      358.573
;            358.573      380.645
;            380.645      426.084
;            426.084      564.241
;            564.241      9033.28
;      
;      IDL> help, *(grs_spectra[50]).spec
;      <PtrHeapVar3170>
;                      FLOAT     = Array[5, 64]
;      IDL> help, *(grs_spectra[50]).grp_drm
;      <PtrHeapVar3172>
;                      FLOAT     = Array[64, 500]
;      IDL> help, *(grs_spectra[50]).grp_eout
;      <PtrHeapVar3173>
;                      FLOAT     = Array[2, 64]
;
;
; :Keywords:
;    path  - path to folder with gerry_grs_drmnew.sav file. Not needed if echan provided
;     defaults to $sswdb_smm
;    file  - defaults to 'ALLFLARES.DAT'
;    drmnew - modified share grs montecarlo response
;    eout   - 2 x 476 GRS data bin energy edges in keV
;
; :Author: raschwar, 28-mar-2019
;-
function grs_alldat_process, path = path, file = file, drmnew = drmnew, eout = eout

  default, path, '$sswdb_smm'
  if ~same_data( size(/dim, eout), [2L, 476L]) || ~exist( drmnew ) then begin
    
    drmnew = 0
    grs_drmfile= loc_file( path=path, 'gerry_grs_drmnew.sav', count=gcount)
    eout = 0
    ein = 0
    if gcount eq 0 then message,'No grs drmfile' else restore, grs_drmfile[0]

  endif

  default, file,'ALLFLARES.DAT'
  ; Attached is an ascii file providing spectrally integrated counts spectra of the SMM flares where I have summed up data channels
  ; to make sure that the background-corrected count rate in each channel is significant at  > two sigma.  
  ; Column one gives the center energy (MeV) and two gives a rough estimate of the width, three gives the count rate (cts/MeV-s), 
  ; and four gives the error.  The fifth column is a best fit rate done back in 1994.    
  dfile = loc_file( path = path, file )
  a = rd_tfile( dfile, 5)
  dates = strarr(185)
  groups = replicate( { tstart: 0.0d0, tend:0.0d0, livetime: 0.0, $
    date: '', nchan: 0, spec:ptr_new(), igroup:ptr_new(), grp_drm: ptr_new(), grp_eout:ptr_new()}, 185)
  idate = 0L
  eout1 = get_edges( eout, /edges_1)
  for i =0, 184 do begin
    groups[i].date = a[0, idate]
    nchan           = fix( a[0, idate+1])
    groups[i].nchan = nchan
    spec = float( a[*,idate+2: idate+2+nchan-1] )
    groups[i].spec = ptr_new(spec)
    idate = idate + 2 + groups[i].nchan
    igroup = grs_alldat_group( groups[i], echan = echan, path = path )
    groups[i].igroup = ptr_new( igroup )
    ig = igroup.igroup
    ig[1,*]++
    groups[i].grp_drm = ptr_new( data_grouper( drmnew, ig))
    groups[i].grp_eout  = ptr_new( get_edges(/edges_2, eout1[ get_edges( ig, /edges_1)]))
  endfor
  out = grs_allflares_read( path = path )
;  IDL> help, out,/st
;  ** Structure <2a8c2170>, 3 tags, length=24, data length=20, refs=1:
;  TSTART          DOUBLE           39172677.
;  TEND            DOUBLE           39172709.
;  LIVETIME        FLOAT           29.8980
; Both files are sequential for the same events so just fill in
  groups.tstart = out.tstart
  groups.tend   = out.tend
  groups.livetime = out.livetime
  return, groups
end