;+
; Name: grs_bksub_egrouped_2fits
;
; Purpose: Write FITS files for the background-subtracted, energy-grouped GRS data provided by G. Share
;
; Method: The data were provided in multiple text files. R. Schwartz pulled all of the information
;   together and wrote a save file - grs_grouped_alldat.sav.  We read that in here, and for each
;   group (which is an event), we write a FITS file called smm_grs_bksub_egrouped_yyyymmdd_hhmm_hhmm.fits
;   where yyyymmdd_hhmm_hhmm is the start and end time of the file.
;
; Written: Kim Tolbert 19-Apr-2019
;-


pro grs_bksub_egrouped_2fits

  restore, 'grs_grouped_alldat.sav'  ; this restores groups and ein
  ng = n_elements(groups)

  for i=0,ng-1 do begin
    data = groups[i]

    nchan = data.nchan
    ntime = 1

    tr = [data.tstart, data.tend]
    stime = time2file(tr[0], sec=0)
    etime = time2file(tr[1], sec=0)

    outdir = '/data/smm/grs/bksub_egrouped/' + strmid(stime,0,4) + '/'
    if ~is_dir(outdir) then file_mkdir, outdir
    outfile = 'smm_grs_bksub_egrouped_' + stime + '_' + strmid(etime,9,4) + '.fits'
    outfile = outdir + outfile

    specnum = 1
    channel = lindgen(nchan)
    timedel = tr[1]-tr[0]
    timecen = avg(tr)
    timezero = anytim(tr[0], /MJD )
    mjd = anytim(tr, /mjd)
    timezero = mjd[0]
    timezeri = mjd[0].mjd    ; Integer part of reference time minus mjdref
    tstartf = double(mjd[0].time)/8.64d7   ; fractional part of ref time, msec / total msec in day
    timezerf = 0.0
    ; Final time of data collection, integer and fractional parts.
    tstopi = mjd[1].mjd
    tstopf = double(mjd[1].time) / 8.64d7

    ct_edges = *data.grp_eout ; count edges in keV
    dspec = *data.spec
    ; original was in counts/MeV/sec, convert to counts/sec units
    ewid = get_edges(ct_edges, /width) / 1000.  ; get energy bin widths in MeV
    cmb = reform(dspec[2,*]) * ewid
    ecmb = reform(dspec[3,*]) * ewid
    nen = n_elements(cmb)
    ltime = data.livetime + fltarr(nen) ; in sec, and now an array of correct dim


    rate_struct = default_rate_header()
    rate_struct.telescope = 'SMM'
    rate_struct.instrument = 'GRS'
    rate_struct.object = 'Sun'
    rate_struct.origin = 'SMM'
    rate_struct.author = 'SPECTRUM2FITS'
    rate_struct.detchans = nchan
    rate_struct.timezero = timezeri - rate_struct.mjdref
    rate_struct.tstarti = timezeri - long(rate_struct.mjdref)
    rate_struct.tstartf = tstartf
    rate_struct.tstopi = tstopi - long(rate_struct.mjdref)
    rate_struct.tstopf = tstopf
    rate_struct.telapse = double(((tstopi-timezeri) + (tstopf-tstartf))*86400.0)

    fxhmake, hdr, 0, /extend, /date
    fxaddpar, hdr, 'DATE_OBS', anytim(tr[0], /ccsds)
    fxaddpar, hdr, 'DATE_END', anytim(tr[1], /ccsds)
    fxaddpar, hdr, 'ENERGY_L', min(ct_edges), 'keV'
    fxaddpar, hdr, 'ENERGY_H', max(ct_edges), 'keV'
    fxaddpar, hdr, 'TELESCOP', 'SMM'
    fxaddpar, hdr, 'INSTRUME', 'GRS'
    fxaddpar, hdr, 'OBJECT', 'Sun'

    primary_hdr = hdr
    fxaddpar, primary_hdr, 'COMMENT', 'Background-subtracted count rates grouped
    fxaddpar, primary_hdr, 'COMMENT', 'in energy from G. Share'

    fxbhmake, hdr, ntime, /date
    ext_hdr = hdr

    ; write spectrum and energy edges as first and second extensions in FITS file
    spectrum2fits, outfile, RATE_STRUCT=rate_struct, /WRITE_PRIMARY_HEADER, $
      PRIMARY_HEADER=primary_hdr, EXTENSION_HEADER=ext_hdr, $
      DATA=cmb, ERROR=ecmb, $
      UNITS='counts/sec', SPEC_NUM=specnum, CHANNEL=channel, $
      TIMEDEL=timedel, $
      TIMECEN=timecen, $
      NROWS = N_elements( timecen ), $ ;added 24-sep-2003 to remove ambiguity
      LIVETIME=ltime, NUMBAND=nchan, MINCHAN=Lindgen(nchan), $
      MAXCHAN=Lindgen(nchan)+1, $   ; S.Bansal-08/31/04-added 1 to each element
      E_MIN=Reform( ct_edges[0,*] ), $
      E_MAX=Reform( ct_edges[1,*] ), E_UNIT='keV', $
      ERR_CODE=err_code, _EXTRA=extra_keys, ERR_MSG=err_msg

    drm_struct = {drm: *data.grp_drm, ein: ein , eout: ct_edges}
    drm_hdr = ext_hdr
    fxaddpar, drm_hdr, 'EXTNAME', 'DRM'
    fxaddpar, drm_hdr, 'TUNIT2', 'keV'
    fxaddpar, drm_hdr, 'TUNIT3', 'keV'
    fxaddpar, drm_hdr, 'COMMENT', 'Response Matrix'
    mwrfits, drm_struct, outfile, drm_hdr
  endfor

end