;+
;
; PROJECT:
; RHESSI
;
; NAME:
; soxs_readfile
;
; PURPOSE: Read SOXS data file

; PREVIOUSLY (before 8-nov-2006):
; Read a single .out soxs file and return data and time array
; The soxs files are text files containing a bunch of rows that look like:
; record counter, hour, min, sec, msec, 256 energy channels
; We're rejected the first two and last two energy bins (Rajmal's instructions)
; We're taking the time to be the start of the accumulation.  There are two
; accumulation modes - 3s and .1s, but we found times that are not spaced by
; 3 or .1 s, so we've eliminated those data points.

; AFTER 8-Nov-2006:
; Read the SOXS .les binary file.  Code to read this binary file was supplied by PRL.
; Files are named either soxs_dd-mmm-yyyy.les (or sometimes on their web site, just
; dd-mmm-yyyy.les) for the Si detector, and  cztsoxs_dd-mmm-yyyy.les for the CZT
; detector.  The first time in the file is used as a base time, and
; subsequent times are generated by adding 3 s or 100 msec depending on the mode of the
; data (given by b(3)).  As before, reject first 2 and last 2 energy bins.

; The energy bin edges are not contained in the input file.  Later we call soxs_drm and
; that returns the energy edges.
;
; CATEGORY:
; SPEX
;
; CALLING SEQUENCE:
; soxs_readfile, file, data, ut_edges, timeshift=timeshift, $
;	err_code=err_code, _extra=_extra
;
; INPUTS:
; FILE - Name of file to read (with path)
;
; Input keywords:
; timeshift - number of seconds to shift data array by
;
; OUTPUTS:
; data - data array (nenergy x ntime)
; ut_edges - time array (2 x ntime) in seconds since 79/1/1
;
; OUTPUT KEYWORDS:
; detector - string containing 'Si' or 'CZT' for detector
; err_code - 0/1 indicates success / failure
;
; EXAMPLE:
; soxs_readfile, f, data, ut, timeshift=90.
;
; Written: Kim Tolbert, June 2006
;
; Modification History:
;	21-Aug-2006, Kim.  Use break_file instead of file_basename for <IDL 6.0 use.
;	31-Oct-2006, K.J.Shah  and A.B. Shah - Read .les file directly instead of
;	  ASCII .out file.  This will handle both Si and CZT files.
;	2-Nov-2006, Kim.  Skip writing .out file and reading it back in.
;	3-Nov-2006, Kim.  Replace a few do loops with vector operations,
;	  use previous value of b(3) to decide whether to add 3 s or 100 msec.
;	6-Nov-2006, Kim.  Added detector output keyword
;	30-Apr-2006,Kim.  Commented out 65536 correction - byte*256.+byte can never be negative.
;-
;------------------------------------------------------------------------------

pro soxs_readfile, file, data, ut_edges, detector=detector, timeshift=timeshift, $
	err_code=err_code, $
	CZT = czt, $
	_extra=_extra

checkvar, timeshift, 0.

err_code = 1
data = -1
ut_edges = -1

print, 'Reading file ' + file

; Date of data is in file name in ddmmyy format.  Convert that to intarr(3)
; so we can use /ext time format (intarr(7) containing [hh,mm,ss,msec,dd,mm,(yy)yy]
; File has hh mm ss msec in columns 1 to 4, so append dd mm yy for each to get time array

break_file, file, disk, dir, name, ext

adet=ssw_strsplit(name,'_',/Head)
detector = strpos(strlowcase(adet),'czt') ne -1 ? 'CZT' : 'Si'
if keyword_set(czt) then begin
	detector = 'CZT'
	adet='cztsoxs'
	endif
if detector eq 'CZT' then adet = 'cztsoxs'
dmy = ssw_strsplit(name, '_', /tail)
date_ext = (anytim(dmy,/ext))[4:6]


;-------------------------------------------------------------------------------------
;	INSERT SOXS DATA CONVERSION PROGRAM TO READ .LES FILES
;
; To generate time of SOXS data according to mode(ie. 3 sec. for S.M.& Q.M.
;	and 100 millisec for Flare Mode along with PHA 0-255 channel
; Which is accepted in OSPEX Program
;	K. J. Shah    time_pha_ospex_kjs.pro


	b={ lbtinf, millsec: lonarr(1), grtsec: lonarr(1), obt_sec: lonarr(1), pid_val: fltarr(50)}

	a={ hbt,fs: bytarr(6), pyt_data: bytarr(700), datapkt_status: intarr(5), $
		dirpid: {lbtinf}, dummy: bytarr(96), frm_grt: bytarr(6), year: bytarr(2), $
		cur_pid_val: {lbtinf}, pkttype: bytarr(4)}


	sid1=lonarr(256)
	b=bytarr(700)
	czd=lonarr(256) & id=intarr(4)
	asid=lonarr(20000,256)
	aczd=fltarr(20000,256)
	atime1=lonarr(20000)
	aday=intarr(20000) & ahr=intarr(20000)
	amin=intarr(20000) & asec=intarr(20000)
	msec=intarr(20000)

	openr,in_lun,file, /get_lun

	rec_num=1

	while not eof(in_lun) do begin

		readu,in_lun,a

		atime1(rec_num)=a.dirpid.grtsec
		msec(rec_num)=a.dirpid.millsec

		b[0:699] = a.pyt_data[0:699]

		if(b(3) eq 1) then goto, id1

		if(adet eq 'cztsoxs') then goto, azd1

		i=0
		for j=8,199,2 do begin
			sid1(i)=b(j)*256.0+b(j+1)
			;if(sid1(i) lt 0) then sid1(i)=sid1(i)+65536
			i=i+1
		endfor

		sid1[96:255] = b[200:359]

	;	Storing Si PHA values as function of record number ie. time
	;	in two dimensional Array

		azd1:

;		q = where (sid1 lt 0, count)
;		if count gt 0 then sid1[q] = sid1[q] + 65536
		asid[rec_num,*] = sid1

		i=0
		for j=360,419,2 do begin
			czd(i)=b(j)*256.0+b(j+1)
	 		;if(czd(i) lt 0) then czd(i)=czd(i)+65536
			i=i+1
		endfor

		czd[30:255] = b[420:645]

;		q = where (czd lt 0, count)
;		if count gt 0 then czd[q] = czd[q] + 65536
		aczd[rec_num,*] = czd

		if(rec_num eq 1) then begin

		;   Conversion of GRT seccond into Hr,Minute and second

			ii1=atime1(rec_num)/86400
			aday(rec_num)=ii1+1
			ak1=atime1(rec_num)-ii1*86400.
			ii2=ak1/3600
			ii2=fix(ii2)
			ahr(rec_num)=ii2
			ak2=ak1-ii2*3600
			ii3=ak2/60
			ii3=fix(ii3)
			amin(rec_num)=ii3
			ak3=ak2-ii3*60.
			asec(rec_num)=ak3

			prevb3 = b(3)

		endif else begin

			ahr(rec_num)=ahr(rec_num-1)
			amin(rec_num)=amin(rec_num-1)
			asec(rec_num)=asec(rec_num-1)
			msec(rec_num)=msec(rec_num-1)

			if(prevb3 eq 17 or prevb3 eq 25) then asec(rec_num)=asec(rec_num)+3
			if(prevb3 eq 9) then msec(rec_num)=msec(rec_num)+100

			if(msec(rec_num) ge 1000) then asec(rec_num)=asec(rec_num)+1
			if(msec(rec_num) ge 1000) then msec(rec_num)=msec(rec_num)-1000

			If(asec(rec_num) ge 60) then amin(rec_num)=amin(rec_num)+1
			If(asec(rec_num) ge 60) then asec(rec_num)=asec(rec_num)-60


			if(amin(rec_num) ge 60) then ahr(rec_num)=ahr(rec_num)+1
			if(amin(rec_num) ge 60) then amin(rec_num)=amin(rec_num)-60

			prevb3 = b(3)
		endelse

		rec_num=rec_num+1

		id1:

	endwhile

free_lun,in_lun

;-------------------------------------------------------------------------------------


; use records 1 through rec_num-1, and use all energies except first two and last two
; for czt data, use aczd array, otherwise, asid array
data = adet eq 'cztsoxs' ? transpose(aczd[1:rec_num-1,2:253]) : transpose(asid[1:rec_num-1,2:253])

; want hour,min,sec,msec in 4,nrec array, where nrec = rec_num-2
hmsm = transpose( [ [ahr[1:rec_num-1]], [amin[1:rec_num-1]], [asec[1:rec_num-1]], [msec[1:rec_num-1]]])
; call anytim with array that's dimensioned by 7,nrec where the 7 are hh,mm,ss,msec, day, month, yy
times = anytim( [hmsm, rebin(date_ext, 3, rec_num-1) ])

; shift times by specified number of seconds
times = reform(times) + timeshift

; get times into 2xn edges format (add last again, so have correct # elements - it will
; be removed below when we're checking for valid duration)
ut_edges = get_edges([times,last_item(times)], /edges_2)
width = get_edges(ut_edges, /width)

; only keep points with 3s or .1s duration since we don't know correct duration for others
q = where ( abs(width - .1) lt .001 or abs(width -3.) lt .001)
ut_edges = ut_edges[*,q]
data = float(data[*,q])

err_code=0

end
