;+
; PROJECT: RHESSI
;
; NAME: spex_chi2_map_calcs  
;
; PURPOSE: OSPEX routine to calculate the chisq min plus n parameter values from a chisq vs parameter
;  value arrays.
;   
; EXPLANATION: This is called by the spex_chi2_map_results routine, usually with plusn = 1 or 4
;   to return 1- and 2-sigma uncertainty on parameter. Parameter value is interpolated to the chimin+plusn value.
; 
; CALLING SEQUENCE: 
;   spex_chi2_map_calcs, p, chi, minind, plusn, valid=valid
;  
; INPUT:
;   p = parameter values
;   chi - chisq values found at each value in p
;   minind - index of minimum chisq value
;   plusn - parameter
;
; OUTPUT KEYWORDS:
;  valid - 2-element array of 0 or 1 indicating failure or success for lower/upper sigma
;
; OUTPUT:
;  Return value is 2-element array of lower/upper parameter value at chimin+plusn
;  If either value of chi isn't reached, a message is printed on screen.
;
; WRITTEN: Kim Tolbert, 30-Aug-2010
; 
; MODIFICATIONS:
;
;-
function spex_chi2_map_calcs, p, chi, minind, plusn, valid=valid

errs = ''
erre = ''

chiplusn = chi[minind] + plusn

w = where (chi[0:minind] gt chiplusn, nst)
if nst eq 0 then begin
  xs = p[0]
  errs = 'lower'
endif else begin
  ii = w[nst-1] + [0,1]
  xs = interpol(p[ii], chi[ii], chiplusn)
endelse

w = where (chi[minind:*] gt chiplusn, nen)
if nen eq 0 then begin
  xe = last_item(p)
  erre = 'upper'
endif else begin
  ii = (w[0] + minind) + [-1,0]
  xe = interpol(p[ii], chi[ii], chiplusn)
endelse

valid = [nst gt 0, nen gt 0]

if errs ne '' or erre ne '' then begin
  msg = 'WARNING: Increase your parameter range!  Minimum chi-square plus '+trim(plusn)+ ' '
  if errs ne '' then message, /cont, msg + errs + ' limit never reached.'
  if erre ne '' then message, /cont, msg + erre + ' limit never reached.'
endif

return, [xs,xe]
end