
;+
; NAME:
;	CURVFIT
; PURPOSE:
;	Non-linear least squares fit to a function of an
;	arbitrary number of parameters.
;	Function may be any non-linear function where
;	the partial derivatives are known or can be approximated.
; CATEGORY:
;	E2 - Curve and Surface Fitting
; CALLING SEQUENCE:
;	YFIT = CURVFIT(X,Y,W,A,SIGMAA,NITER=NITER,FREE=FREE,CHISQR=CHISQR,$
;			RANGE=RANGE)
; INPUTS:
;	X = Row vector of independent variables.
;	Y = Row vector of dependent variable, same length as x.
;	W = Row vector of weights, same length as x and y.
;		For no weighting
;		w(i) = 1., instrumental weighting w(i) =
;		1./y(i), etc.
;	A = Vector of nterms length containing the initial estimate
;		for each parameter.  If A is double precision, calculations
;		are performed in double precision, otherwise in single prec.
; Keyword Parameters:
;	Function_Name = Name of function (actually, a procedure) to fit.
;		If omitted, use "FUNCT". The procedure must be written as
;		described below.
;	Niter = number of loops for calculating curvature matrix
;	Free  = index describing parameters, 1=free, 0=fixed
;	Nolambda - if set, then fit is obtained with FLAMBDA set to zero
;	Chisqr - Value of reduced chisquare when curvefit is exited
;	Range - Maximum and minimum values allowed for free parameters
; OUTPUTS:
;	A = Vector of parameters containing fit.
;	Function result = YFIT = Vector of calculated
;		values.
; OPTIONAL OUTPUT PARAMETERS:
;	Sigmaa = Vector of standard deviations for parameters
;		A.
;	
; CALLS TO:
;	SPEX_FCHISQR, CHECKVAR
; COMMON BLOCKS:
;	NONE.
; SIDE EFFECTS:
;	The function to be fit must be passed 
;	as FUNCTION_NAME.  It and its partial derivatives
;	are calculated inside SPEX_FCHISQR using a call to the procedure
;	FUNCT1.  This procedure accepts values of
;	X (the independent variable), Function_name 
;	and A (the fitted function's
;	parameter values), and returns F (the function's values at
;	X), and PDER (a 2D array of partial derivatives).
;	Call to FUNCT1 is:
;	FUNCT1,	FUNCTION_NAME, X, A, F, PDER              
;
; where:
;	FUNCTION_NAME - Name of the fitting function which accepts X & A.
;	X = Vector of NPOINT independent variables, input.
;	A = Vector of NTERMS function parameters, input.
;	F = Vector of NPOINT values of function, y(i) = funct(x(i)), output.
;	PDER = Array, (NPOINT, NTERMS), of partial derivatives of funct.
;		PDER(I,J) = DErivative of function at ith point with
;		respect to jth parameter.  Optional output parameter.
;		PDER should not be calculated if parameter is not
;		supplied in call (Unless you want to waste some time).
; RESTRICTIONS:
;	NONE.
; PROCEDURE:
;	Copied from "CURFIT", least squares fit to a non-linear
;	function, pages 237-239, Bevington, Data Reduction and Error
;	Analysis for the Physical Sciences.
;
;	"This method is the Gradient-expansion algorithm which
;	compines the best features of the gradient search with
;	the method of linearizing the fitting function."
;
;	Iterations are perform until the chi square changes by
;	only 0.1% or until 20 iterations have been performed.
;
;	The initial guess of the parameter values should be
;	as close to the actual values as possible or the solution
;	may not converge.
;
; MODIFICATION HISTORY:
;	Written, DMS, RSI, September, 1982.
;	Does not iterate if the first guess is good.  DMS, Oct, 1990.
;	Added CALL_PROCEDURE to make the function's name a parameter.
;		(Nov 1990)
;	Added call to SPEX_FCHISQR. ras, apr, 1992
;	fixed iarray problem, ras, 11-aug-94
;	changed array divides to f_div to protect against 0's	
;-
;	ON_ERROR,2		;RETURN TO CALLER IF ERROR
		
FUNCTION CURVFIT,X,Y,W,A,SIGMAA, Function_Name = Function_Name, NITER=NITER, $
	FREE=FREE, QUIET=QUIET, NOLAMBDA=NOLAMBDA, CHISQR=CHISQR, $
	RANGE = PRANGE
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;DEFINE FITTING FUNCTION          
	
	;Name of function to fit
	checkvar, function_name, 'f_vth_bpow'
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	CHECKVAR, NITER, 20

	A = 1.D0*A		;MAKE PARAMS DOUBLE PRECISION
	NTERMS = N_ELEMENTS(A)	;# OF PARAMS.
	CHECKVAR, FREE, INTARR(NTERMS)+1 ;ASSUME ALL ARE FREE PARAMETERS
	CHECKVAR, PRANGE, $
		[[FLTARR(NTERMS)+1e-20],[FLTARR(NTERMS)+1e+20]]
	NTERMS = FIX(TOTAL(FREE)) ;NUMBER OF FREE PARAMETERS
	NFREE = (N_ELEMENTS(Y)<N_ELEMENTS(X)) - NTERMS ;Degs of freedom
	WF    = WHERE(FREE EQ 1,NWF) ;INDICES OF FREE PARAMETERS
	IF NFREE LE 0 THEN STOP,'Curvefit - not enough data points.'

	IF KEYWORD_SET(NOLAMBDA) THEN LAMBDA_SCALE=0 ELSE LAMBDA_SCALE=1
	FLAMBDA = 0.001	* LAMBDA_SCALE	;Initial lambda
	DIAG = INDGEN(NTERMS)*(NTERMS+1) ;SUBSCRIPTS OF DIAGONAL ELEMENTS
;
	FOR ITER = 1,NITER DO BEGIN	;Iteration loop
;
;		EVALUATE ALPHA AND BETA MATRICIES.
;
 	CHISQ1 = SPEX_FCHISQR( X,Y,W,Function_name,A,YFIT,PDERF,free=Wf )/NFREE
;	PDERF = REFORM(PDER(*,WF),N_ELEMENTS(Y),NWF) ;PARTIAL DERIVATIVES OF FREE PARAMETERS
	BETA = (Y-YFIT)*W # PDERF
	ALPHA = TRANSPOSE(PDERF) # (W # (FLTARR(NTERMS)+1)*PDERF)

;'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
; For some reason DMS included these statements which make no sense!!!
; Commented out by RAS, 92/09/29
;
; If a good fit, no need to iterate
;	if chisq1 lt total(abs(y))/1e7/NFREE then begin
;		sigmaa = fltarr(nterms)	;Return all 0's
;	     	return, yfit
;		endif
;'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
;
;	INVERT MODIFIED CURVATURE MATRIX TO FIND NEW PARAMETERS.
;
CURVATURE: 
		C = SQRT(ALPHA(DIAG) # ALPHA(DIAG))
		;ARRAY = ALPHA/C
		ARRAY = f_div(ALPHA, C)
		DB = A*0.0 			;INITIALIZE INCREMENTS
		B = A
		ARRAY(DIAG) = 1.0 + FLAMBDA
		IF NTERMS GT 1 THEN IARRAY = INVERT(ARRAY) ELSE IARRAY=f_div(1.,ARRAY)      ;ras 11-aug-94
		DB(WF) = f_div(IARRAY,C) # TRANSPOSE(BETA) ;NEW PARAMS
		B(WF)= ((A(WF) + DB(WF)) > PRANGE(WF,0)) < PRANGE(WF,1)
		CHISQR= SPEX_FCHISQR(X,Y,W,Function_name,B,YFIT)/NFREE ;EVALUATE FUNCTION
		FLAMBDA = FLAMBDA*10.	;ASSUME FIT GOT WORSE

		IF CHISQR GT CHISQ1 AND LAMBDA_SCALE NE 0 THEN GOTO, CURVATURE 
	                

	FLAMBDA = FLAMBDA/100.	;DECREASE FLAMBDA BY FACTOR OF 10
	A= B 			;SAVE NEW PARAMETER ESTIMATE.

;	goto, skip
	IF ((CHISQ1-CHISQR)/CHISQ1) LE 1E-5 AND LAMBDA_SCALE EQ 1 THEN BEGIN
		;FINISH WITH CURVATURE FIT
		print,'USE CURVATURE MATRIX WITH FLAMBDA=0, CHISQ=',$
			CHISQR
		YFIT = CURVFIT(X,Y,W,A,SIGMAA,FUNCTION_NAME=FUNCTION_NAME, $
			QUIET=QUIET,/NOLAMBDA,FREE=FREE, range=prange,NITER=1, $
			CHISQR=CHI_CURVATURE)
		IF CHI_CURVATURE LE CHISQR THEN BEGIN
			PRINT,'CHISQR, CHI_CURVATURE'
			PRINT, CHISQR, CHI_CURVATURE
			CHISQR=CHI_CURVATURE 
		ENDIF ELSE A=B ;IF CURVATURE ISN'T BETTER, THEN USE PREVIOUS A
		GOTO,DONE ;Finished
	ENDIF
	skip:
	ENDFOR			;ITERATION LOOP
;
	message, 'No improvement using analytic expansion, i.e lambda=0',/INFORMATIONAL
;

DONE:   SIGMAA = SQRT(f_div(ARRAY(DIAG),ALPHA(DIAG))) ;RETURN SIGMA'S
	IF NOT KEYWORD_SET(QUIET) THEN BEGIN
  		;stop
		PRINT,'ITERATION =',ITER,' ,CHISQR =',CHISQR
		PRINT,A
		print,'FLAMBDA=',flambda
	ENDIF
	RETURN,YFIT		;RETURN RESULT
END
