
;+
;
; NAME: FILE_SEARCHER
;
;
; PURPOSE: Look for files satisfying passed directory and filename strings,
;	   Find file or files satisfying passed strings, and let user select filename
;	   of choice.  Passed strings may also be a set of templates using wildcards
;	   and replacement values
;
; CATEGORY: i/o
;
;
; CALLING SEQUENCE: file_searcher, templates=templates, wildcard=wildcard, spex_test_dir=spex_test_dir, $
;	inserts=inserts, out_fname=out_fname, out_dir=out_dir, nointeract=nointeract
;
; CALLED BY: SPEX menu
;	
;
; CALLS TO:
;	BUILD_FNAME, LOC_FILE, XSEL_LIST, PRINTX
;
; INPUTS:
;	templates - possible formats for filename including wildcards for
;		    substitution (see build_fname.pro)
;	spex_test_dir  - directory(ies) to be combined with templates using concat_dir
;	(see build_fname.pro for explanations of the following optional arguments)
;	wildcard  - string to be substituted for in templates
;	inserts   - substitution strings for wildcard in templates for test_file
;	/nointeract- if set, then user is NOT asked to select if multiple files are found.

; OUTPUTS:
;	Files - files found satisfying inputs.
; KEYWORDS:
;       out_fname - filename satisfying input specification, '' if none
;       out_dir   - directory satisfying input specification, '' if none
;
; PROCEDURE: uses build_fname and findfile to test strings in filename searches.
;	     if more than one is returned, requests the user to specify interactively.
;            If templates are passed, then they are checked first before test_file and spex_test_dir
; RESTRICTIONS:
;	Developed for SPEX, but being modified for general usage.
; MODIFICATION HISTORY:
;	Version 1, ras, 1992
;	Version 5?, ras, 18-oct-1996, using loc_file and xsel_list
;	Version 6, richard.schwartz@gsfc.nasa.gov, don't select unless there are
;	really 2 or more files.
;	Version 7, richard.schwartz@gsfc.nasa.gov, fix conflict with test_dir()
;-
pro file_searcher, files, templates=template, wildcard=wildcard, spex_test_dir=spex_test_dir, $
	inserts=inserts, out_fname=out_fname, out_dir=out_dir, nointeract=nointeract

;print, 'template'
;print,template

;print, 'inserts'
;print, inserts

;print,'spex_test_dir'
;print,spex_test_dir

files = ['']
n_autotest = n_elements(template)
checkvar, spex_test_dir, ''
;for j=0,n_elements(spex_test_dir)-1 do begin
for i=0,n_autotest-1 do begin
  if n_elements(inserts) eq 0 then insert='' else insert=inserts(*,i)
  checkvar,wildcard,'&'
  if strpos( template(i), wildcard) ne -1 then $
    test_file = build_fname( template(i), wildcard, insert) else $
    test_file = template(i)
  ;test = concat_dir(spex_test_dir(j),test_file)
  ;printx,string(/;print, 'In file_searcher, looking for ',test)
  if strcompress(/rem,test_file) ne '' then $
    found_files = loc_file(/all,test_file,path=spex_test_dir,count=nfiles) else nfiles=0
  if nfiles eq 1 then begin
    if strpos(strupcase(found_files(0)),strupcase(test_file)) ne -1 then begin
      printx, 'Found this file: '+ found_files
      files = [files, found_files(0)]
      goto, leave_find 
      endif else files = [files, found_files]
  endif
  if nfiles gt 1 then files = [files, found_files]
endfor
;endfor
leave_find:

files  = files( uniqo(files))
wfiles = where( files ne '', nfiles)
if nfiles ge 1 then begin
files  = files( wfiles )
case 1 of 
	nfiles eq 1: files=files
	nfiles ge 2: begin
	if not keyword_set(nointeract) then begin
		if f_use_widget(/test) then $
		   nselect = xsel_list( files, /index, title='Select the datafile below') $
		else $
		   nselect = emenu( files, $
			mtitle='Enter the number for the desired datafile from the following:', $
			imax = 100 < n_elements(files) )
		files = files(nselect)
	endif else printx,'Multiple filenames returned!  Possible error.'
	end
endcase
endif else files = ''

out_fname=''
out_dir  =''
wfiles = where( files ne '', nfiles)
if nfiles eq 1 then begin
	break_file, files(wfiles(0)), tdisk, tdir, tfil, texten, tvers
	out_fname = tfil + texten +tvers
	out_dir   = tdisk+tdir
endif
return
end
