function near_fwhm, e, par
return, e*0.0 + par(0)
end

;+
;
; NAME: 
;	NEAR_PIN_RESP
;
; PURPOSE:
;	This procedure computes the X-ray response of the NEAR P-I-N SI detector to a solar flux.
;	Because it is designed to work with the generic SPEX
; CATEGORY:
;	NEAR, SPEX
;
; CALLING SEQUENCE:
;	NEAR_PIN_RESP,  EDGES_OUT=EDGES_OUT, EDGES_IN=EDGES_IN, $
;		AREA=AREA, DRM=DRM
;
; CALLS:
;	RESP_CALC, XSEC, NEAR_PIN_CAL, F_DIV, DATATYPE, PULSE_SPREAD
;
; KEYWORDS:
;	EDGES_OUT- Energy channel edges in keV.
;	EDGES_IN - Photon input energies for DRM in keV.
;	DRM - detector response, counts/sec/cm2/keV per photon/keV/cm2.
;	DIFF_COS_THETA - set number of photon inputs to 600 up to 25 keV
;	THETA- Response to a beam at a zenith angle of theta.
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;       none explicit, only through commons;
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	none
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	none
;
; MODIFICATION HISTORY:
;	Version 1	ras, 17-may-1996	
;	Version 2 	ras, 15-aug-1996, added to common block, changed interpolation
;			range
;	Version 3	ras, 24-jan-1997, near distance now scales area
;			concommitant changes made to rd_near_pin.pro
;	Version 4, RAS, 16-Mar-1997, added newvalues structure to common block
;	Version 5, RAS, 1-apr-1997, encoded newvalues, although not as defaults yet
;-

PRO NEAR_PIN_RESP, EDGES_OUT=ENEAR, EDGES_IN=EDGES, AREA=AREA, DRM=DRM, $
_extra=_extra, diff_cos_theta=diff_cos_theta, theta=theta, nflux=nflux,$
ehi=ehi

common near_pin_resp, gain_mod, edges_out_sav, edges_in_sav, eloss_mat, newvalues

;old values used prior to 1-apr-1997
depleted_microns = 270.0
dead_layer_microns = 0.3
tailing_constant = 0.24
;new values based on Richard Starr's lab calibration of engineering model
if keyword_set(diff_cos_theta)   then begin
    
    endif

if keyword_set( newvalues) then case 1 of
    datatype(newvalues) eq 'STC':  begin
        tags = tag_names( newvalues)
        for i=0,n_tags(newvalues) -1 do test = execute( tags(i)+'=newvalues.(i)')
        end
    else: begin
        depleted_microns = 270.0
        dead_layer_microns = 0.3
        tailing_constant = 0.25 ; invariant with energy.
        newvalues = { depleted_microns:depleted_microns,$
        dead_layer_microns:dead_layer_microns, $
        tailing_constant: tailing_constant}
        end
    endcase

if n_elements(gain_mod) eq 0 then gain_mod= {gain, gain: 1.0, offset:0.0, fwhm: 0.580}


if datatype( gain_mod ) eq 'STC' then begin
    tags = tag_names(gain_mod)
    w_fwhm = (where( tags eq 'FWHM', nfwhm))(0)
    w_gain = (where( tags eq 'GAIN', ngain))(0)
    w_offset = (where( tags eq 'OFFSET', noffset))(0)
    endif
if datatype( eloss_mat ) eq 'STC' then begin
    tags = tag_names( eloss_mat)
    w_fwhm = (where( tags eq 'FWHM', mfwhm))(0)
    w_gain = (where( tags eq 'GAIN', mgain))(0)
    w_offset = (where( tags eq 'OFFSET', moffset))(0)
    endif

common f_vth_com, use_vlth
use_vlth = 1

detector='SI
;
;Allow for area to be initialized by the data reader which includes information
;on the distance of the spacecraft from the Sun in Astronomical Units.
checkvar,area,  .0121 ;cm2

if keyword_set(mfwhm)  then fwhm_old = eloss_mat.fwhm else fwhm_old =0.580
if nfwhm eq 1 then fwhm = gain_mod.fwhm else fwhm = fwhm_old
func='near_fwhm
func_par=[fwhm,fltarr(3)]
d = depleted_microns *1e-6 *100
elo = 1.001 ;keV
checkvar, nflux, 300
checkvar, ehi, 13.0 ;keV
eout= findgen(221)*.07 + .1
edge_products, eout, edges_2=eout
z=[4,14]
gmcm=[.00762*1.847, dead_layer_microns*1e-4*2.33]
;print,gmcm
;    0.0140741  0.000233000
if n_elements(eloss_mat) eq 0 or fwhm ne fwhm_old or keyword_set(newvalues)  then begin
    if keyword_set( diff_cos_theta) then begin
        ;
        ;  
        ;  
        
        endif 
    resp_calc, detector,area,func,func_par,d,z,gmcm,nflux,elo,ehi, $;inputs
    eloss_mat1, pls_ht_mat, edges, smatrix_mat, eout=eout, theta = fcheck(theta,0.0)
    ;Convolve with a tailing function to represent incomplete charge collection
    ;Our single empirical data point is 19% tailing at 5.9 keV, collected from 1.2-5.7 keV
    ;Tailing is produced through incomplete collection on the sides of
    ;of the detector.  Only the inner 1.1 mm of 1.6 mm is depleted.
    wmat1=where(eloss_mat1 ne 0.0)
    emat1=eloss_mat1(wmat1)
      
    nout = n_elements(eloss_mat1(*,0))
    edge_products, eout, wid=wout, mean=emout
    wij = get_ij( where( eloss_mat1(1:*,*) gt 0.0), nout-1 )
    wij(0,*) = wij(0,*)+1
    for i=1,nflux-1 do  begin
	w = where( wij(1,*) eq i, nw)
	for j=0,nw-1 do $
	eloss_mat1(0,i)=eloss_mat1(*,i) + $
	tailing_constant*wout(0:wij(0,w(j))-1)*eloss_mat1(wij(0,w(j)),i)/emout(wij(0,w(j)))
    endfor
    
    
    
    endif else edges = edges_in_sav

edge_products, edges, edges_2=edges, wid=wedges, mean=emdges

if keyword_set(mgain) then gain_old = eloss_mat.gain else gain_old=1.0
if ngain eq 1 then gain = gain_mod.gain else gain = gain_old
if keyword_set(moffset) then offset_old = eloss_mat.offset else offset_old=0.0
if noffset eq 1 then offset = gain_mod.offset else offset = offset_old

if n_elements(eloss_mat1) ge 1 then $
eloss_mat = { eloss_mat:eloss_mat1, $
fwhm: fwhm, gain: gain, offset:offset, eout:eout, wmat:wmat1, emat:emat1} else begin
    eloss_mat.fwhm = fwhm
    eloss_mat.gain = gain
    eloss_mat.offset = offset
    endelse
gain_mod= {gain, gain: gain, offset:offset, fwhm: fwhm}
eout = eloss_mat.eout
edge_products, eout, wid=wout, mean=emout
nout = n_elements(emout)

near_pin_cal, enear, gain, offset

edge_products, enear, edges_2=enear, mean=emnear, wid=wnear
edges_out_sav = enear
nnear = n_elements(emnear)

input_psm = {ein: eout, eout: enear, func:func, func_par:func_par}

pulse_spread, input_psm, pulse_shape, eloss_mat.eloss_mat, drm

drm = f_div( drm, rebin(wnear, nnear, nflux)) / area

checkvar, edges_in_sav, fltarr(2,nflux)
if total( abs(edges_in_sav-edges) ) ne 0 then begin
    ;help,vlth( 1., 1., te6=exp(interpol(alog([1.0,20.0]),400)), edges= edges)
    edges_in_sav  = edges
    endif

end
