;+
;
; NAME: 
;	RD_NEAR_PIN
;
; PURPOSE:
;	This procedure ingests the data from the NEAR PIN detector.
;	Times and fluxes are corrected to distances of 1 AU.
; CATEGORY:
;	NEAR, SPEX
;
; CALLING SEQUENCE:
;	RD_NEAR_PIN, FILES=FILES, FLUX=FLUX, EFLUX=EFLUX, $
;	UT=UT, UNITS=UNITS, AREA=AREA, LTIME=LTIME, EDGES=EDGES
;
; CALLS:
;	NEAR_PIN_CAL, EDGE_PRODUCTS, BREAK_FILE, RD_TEXT, ANYTIM, F_DIV
;
; KEYWORDS:
;  INPUTS:
;       FILES - Filename
;  OUTPUTS:
;	FLUX  - NEAR PIN flux in cnts/cm2/sec/keV corrected to distance of 1 AU
;		256 x Number_of_intervals
;	UT    - Start and end time of integration intervals corrected to 1 AU in sec from
;	        1-jan-1979, 2 x Number_intervals
;	EFLUX - Poisson uncertainties on Flux
;	AREA  - Detector area geometric area
;	LTIME - Live time, fltarr(number_of_intervals)
;	EDGES - 2x256 lo and hi energy edges in keV
;	WCHAN - default channels
; COMMON BLOCKS:
;	near_pin_resp
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	none
;
; MODIFICATION HISTORY:
;	Version 1	ras, 17-may-1996
;       Version 2       ras, 19-July-1996, fixed ltime division bug	
;	Version 3	ras, 15-Aug-1996, always calculate enear
;	Version 4	ras, 19-nov-1996, use near_pin_cal
;	Version 5	ras, 4-dec-1996, use rd_text
;	Version 6	ras, 24-jan-1997, near distance now scales area
;			concommitant changes made to near_pin_resp.pro
;	Version 7 ras, 16-Mar-1997, added newvalues to common block
;	version 8, richard.schwartz@gsfc.nasa.gov
;	force times into row vector, 3-nov-1997
;-
pro RD_NEAR_PIN, FILES=FILES, FLUX=FLUX, EFLUX=EFLUX, $
	UT=UT, UNITS=UNITS,  AREA=AREA, $
	LTIME=LTIME, EDGES=EDGES, $
	WCHAN=WCHAN, $
;	DUMMY PARAMETERS!!! FOR VERSION 3.6c
	start_time = start_time, end_time=end_time, flare=flare, $
	det_id=det_id, id_style=id_style , id_use=id_use,  $	
	auto=auto, noplot=noplot,  title=title

                    


common near_pin_resp, gain_mod, edges_out_sav, edges_in_sav, eloss_mat, newvalues

if n_elements(gain_mod) eq 0 then gain_mod= {gain, gain: 1.0, offset:0.0, fwhm: 0.5}

au = 1.495978d13 ;cm
cl = 2.99792458d10 ;cm/s


if datatype( gain_mod ) eq 'STC' then begin
	tags = tag_names(gain_mod)
	w_gain = (where( tags eq 'GAIN', ngain))(0)
	w_offset = (where( tags eq 'OFFSET', noffset))(0)
endif
if ngain eq 1 then gain = gain_mod.gain else gain = 1.0
if noffset eq 1 then offset = gain_mod.offset else offset = 0.0

near_pin_cal, enear, gain, offset

edge_products, enear, edges_2=enear, mean=emnear, wid=wnear
edges_out_sav = enear

edges = edges_out_sav
edge_products, edges, wid=wedges
chan_width = wedges(100) ;linear gain/channel, use it!

break_file,files(0),disk,dir,fnam,ext
case 1 of 
	strlowcase(ext) eq '.txt': begin
  
	near_txt=rd_text(files(0))
	vals = ['times','date','ltime','dtime','rnear']
	times =0
	date  =0
	rnear =0
	ltime =0
	dtime =0
	for i=0,n_elements(vals)-1 do $
	  r = execute( near_txt((where( strpos(near_txt,vals(i)) eq 0))(0)))
	lst = (where( strpos(near_txt,'rnear') eq 0))(0) + 2
	nbins = n_elements(dtime)	
	nearc=fltarr(nbins,256)
	area = 0.0121 / rnear^2
	reads,near_txt(lst:lst+255),nearc
	nearc = transpose(nearc)
	flux= nearc / area
	flux = flux/chan_width 
	;if times is in seconds, the input must be double precision to assure
	;a normal row vector, so force a row vector.
	time = reform(anytim(/utime, times))
	time = [time,time+dtime]
	time = transpose( reform(time,nbins,2))
	ut= anytim(/utime, time) + anytim(/utime, date) - (rnear-1.0)*au/cl
	eflux = (sqrt( nearc > 1)/( chan_width *area))
	flux  = f_div( flux, (1.+fltarr(256))#ltime )
	eflux = f_div( eflux,(1.+fltarr(256))#ltime )
	end
	strlowcase(ext) eq '.sav':  restore,file=files(0), /verb
	else: message,/cont,'Warning, no data read'
endcase
units=' s!u-1!n cm!u-2!n keV!u-1!n'
wchan = (indgen(256))(30:240)
title='NEAR PIN  '+atime(/yoh,ut(0),/date)

end
