;+
; NAME
;	RD_TGRS
; PURPOSE
; 	Read in spectral accumulation files produced by the 
;	TGRS Data Analysis Software (TDAAS).
;	Suitable for reading ANY TGRS data produced
;	by the routine COUNT in TDAS.
;
; INPUTS
;	Filter- full file name including path and wildcards.  Integrates
;	all these files into a single output
; OUTPUTS
;	FLUX  - count rate in counts/sec/cm2/keV
;	EFLUX - Poisson uncertainty on Flux
;	EDGES - energy edges 2xNumber_of_channels
;	UT    - Start and end time in seconds from 1 jan 1979
; KEYWORDS
;	ERROR- If spectrum is not read properly, error will be set to 1.
;	BTYPE- If set, then background is already subtracted, 'BSB' file
; CATEGORY
;	TGRS, SPEX
; RESTRICTIONS -
;	The file does encode the x-axis and y-axis types, but we are assuming
;	that everything is in energy units for the moment.
;	Only use on VMS.
; HISTORY
;	RAS, 23 - JULY - 1996, ADAPTED FROM RD_SPEC.PRO
;
;-
pro rd_tgrs, filter, ut, edges, flux, eflux, error=error, btype=btype

error = 0

f = findfile(filter,count=count)
if count eq 0 then goto, error_exit

rd_tgrs_spec,f(0), uti, edges, fluxi, efluxi, btype=btypei
ut = dblarr( 2, count)
ut(0,0) = uti
flux = reform(fluxi) #(fltarr(count)+1.) 
eflux = reform(efluxi) #(fltarr(count)+1.) 
btype = intarr(count)+btypei

for i=0,count-1 do begin
	rd_tgrs_spec,f(i), uti, edges, fluxi, efluxi, btype=btypei, error=error
	if error then goto, error_exit
	ut(0,i) = uti
	flux(0,i) = reform(fluxi)
	eflux(0,i) = reform(efluxi)
	btype(i) = btypei 
endfor

;Check the order!!
ord = sort(ut(0,*))
ut  = ut(*, ord)
flux  = flux(*, ord)
eflux  = eflux(*, ord)
btype  = btype( ord)

return

error_exit:
error = 1
print,'% RD_SPEC>> Error in reading ',filename


end
 
