;+
; Project     : SDAC   
;                   
; Name        : REDUCE_GOES
;               
; Purpose     : This procedure reads GOES data, subtracts background, integrates over
;		time intervals, reports the fluxes, and converts to temperature and emission measure.
;               
; Category    : GOES
;               
; Explanation : This procedure reads GOES data over a single time range then integrates over
;		selected time intervals after subtracting background.  It then returns temperatures and emission measures
;		for the averaged values.
;               
; Use         : REDUCE_GOES, Timerange, Ut, Goes, Temp, Em, TBACK=TBACK, TIMESEL=TIMESEL
;		[, IGOES=IGOES, ITEMP=ITEMP, IEM=IEM] [,SAT=SAT]
;    
; Inputs      : Timerange - Accumulate 3-second GOES data over this time interval.  Time in ANYTIM readable format.
;		Timesel   - 2xN time intervals, anytim format.               
; Opt. Inputs : None
;               
; Outputs     : Ut
;		Goes
;		Temp
;		Em 
;
; Opt. Outputs: None
;               
; Keywords    :
;
; Calls	      :
;
; Common      : None
;               
; Restrictions: 
;               
; Side effects: None.
;               
; Prev. Hist  :
;
; Modified    : Version 1, RAS, 2-apr-1997
;
;-            
;==============================================================================
pro reduce_goes,   ut,   goes, temp, em, plot=plot, ntemp=ntemp, nem=nem, sat=sat, $
	 timerange=timerange, tback=tback, garcia=garcia, timesel=timesel, masklim=masklim, $
	igoes=igoes, itemp=itemp, iem=iem, tplot=tplot, yrange=yrange, pmulti=pmulti

;if datatype( timerange ) eq 'STC' then begin
;gbo_struct, gxd_data=gxd_data
;case 1 of 
;	(where_struct(timerange.(tag_exist(timerange, 'lo') and tag_exist(timerange,'hi') and $
;	 tag_exist(timerange,'time') and tag_exist(timerange,'day')): begin
	
;	ut = anytim(/sec, timerange)
;	goes = reform([timerange.wuse.lo,timerange.wuse.hi],n_elements(timerange.wuse),2)
;	end
; or 
;	tag_exist(timerange,'goes') 
;	wgoes= wc_where( tags, '*goes*', /case_ignore,count)
;	if count eq 0 then begin
;	wgen = wc_where( tags, 'gen',/case_ign, count)
;	if count eq 1 then begin
;		tags = tag_names(timerange.(wgen(0)))
;		wlo  = where( strp
	goes_data, stime=timerange(0), etime=timerange(1), sat=sat, /clean,$
	utbase=utbase, tarray=ut, yclean=  goes

ut = ut + utbase

r = .08617
checkvar, tplot, 0

if keyword_set( tback ) then begin

	for i=0,1 do begin

	back = 0.0

	for j=0,n_elements( tback(0,*) )-1 do back = back + interp2integ( tback(*,j)-ut(0), ut-ut(0),   goes(*,i))
	  goes(*,i) =   goes(*,i) - back/total( tback(1,*)-tback(0,*))

	endfor

endif

;goes = goes -rebin(reform(avg(goes(0:49,*),0),1,2),n_elements(ut),2)

goes_tem,    goes(*,0),   goes(*,1), temp, em, $
	satellite=satellite, garcia=garcia

temp = temp * r
if keyword_set(timesel) then begin
	igoes = fltarr(2, n_elements( timesel(0,*) ))


	for i=0, n_elements( timesel(0,*) )-1 do for j=0,1 do $
	igoes(j, i) = interp2integ( timesel(*,i)-ut(0), ut-ut(0),   goes(*,j) )/( timesel(1,i)-timesel(0,i))
	goes_tem, igoes(0,*), igoes(1,*), itemp, iem, satellite=satellite, garcia=garcia
endif

if keyword_set(tplot) then begin

if keyword_set(timesel) then begin
 !p.multi=fcheck(pmulti,0)
 mask = temp*0+1.
 mask(where(    goes(*,0) lt fcheck(masklim,1e-7)))=0.0
  utplot, ut,   goes(*,0)*(1-tplot) + tplot * temp*mask, psy=10, timerange=timerange, yrange=yrange

 ycrange = crange('y')
  for i=0,   n_elements( timesel(0,*) )-1 do begin
	oplot, timesel(0,i)+fltarr(2), ycrange
	oplot, timesel(1,i)+fltarr(2),ycrange
	xyouts, timesel(1,i), ycrange(1)*.1+0.9*ycrange(0),orient=90, $
	'T= '+string(itemp(i)*r,form='(f4.2)') +' keV'+ $
	' EM= '+string(iem(i)*r,form='(f7.4)')+ ' 10!u49!n cm!u-3!n',chars=1.5, col=5
  endfor
endif else begin
  !p.multi = [0,1,3]
  utplot, ut,   goes(*,0), psy=10,chars=2, timerange=timerange
  utplot, ut, temp/11.36, psy=10,chars=2,yran=[0,2] , timerange=timerange
  utplot, ut, em, psy=10,chars=2,yran=[0,1], timerange=timerange
endelse
endif
end
;+
; Project     : SDAC
;                   
; Name        : RUN_NEAR
;               
; Purpose     : This main program performs the initial analysis on the 8 May 1996 solar
;	xray observations by the Si PIN detector on the Near Earth Asteroid Rendezvous(NEAR)
;	spacecraft.
;               
; Category    : NEAR, ANALYSIS, SPEX
;               
; Explanation : The initial commands are fed into SPEX.  
;	command_list=['data,near,pin','f_model,f_con_lin','apar,.1,1.,1.,1.,1.,1.,7.5e-5,0.0','range_lo,,.2,,.4', $
;	'_1file,960508_near2.txt','graph','read_drm','back_order,0','back','t_h,c','zoom',$
;	'sele','sp_xty,0','spyr,10,1e6','bspec','enrange','free,1,1,0,0,0,0,0,0','fit','sumfit,1','fit']
;;;;;;;;; START DIALOG TEXT- DO NOT EDIT THIS LINE
;#     Change Option: data,near,pin - The mission and instrument are selected.
;#     Change Option: f_model,f_con_lin - the functional model chosen.
;	F_CON_LIN has an additional attenuation constant (apar(6)), the thickness in
;	cm of Si dead layer not in the response function model.  This function also
;	allows variation in the relative abundances from nominal of the line producing
;	elements Fe, Si, S, and Ca.
;#     Change Option: apar,.1,1.,1.,1.,1.,1.,7.5e-5,0.0 - initial model parameters, note 0.75 microns more of Si.
;	making a total of ~1.0 micron of Si.  Could be other attenuating material.
;#     Change Option: range_lo,,.2,,.4 - lower limits on two of the parameters, temp in keV, and relative Si abundance.
;#     Change Option: directory, path  - directory specifies single path to the data file to read.
;#     Change Option: _1file,960508_near2.txt - _1file specifies the data file to read.
;#     Command: graph - Read and plot the data file in the time history window.
;#     Command:read_drm - Create the detector response matrix and install it as drm.
;#     Change Option: back_order,0 - Choose a background constant in time for each energy.
;#     Command: background - Select the background interval in the TIME HISTORY window.
;	A BACKGROUND SELECTION dialog box will appear to help you. 
;#     Change Option:t_hist_mode,c - Plot the count rate in units of counts/sec in each band under GRAPH.
;#     Command: zoom - Select two boundaries and expand the time historyin the TIME HISTORY window.
;	A ZOOM dialog box will appear to help you. 
;#     Command: select - Select the data accumulation intervals from the TIME HISTORY window.
;	Several SELECT dialog boxes will appear to help you. 
;#     Change Option: sp_xty,0 - Set the spectral plot x axis to linear.
;#     Change Option: spyr,10,1e6 - Set the spectral plot y range.
;#     Command: bspectrum - Plot the bacground spectrum in the spectral window.
;#     Command: enrange - Graphicly choose the fitting energy range in the spectral window.
;	An ENERGY RANGE dialog box will appear to help you.
;#     Change Option: free,1,1,0,0,0,0,0,0 - For the fitting model, 1 means vary this parameter.
;#     Command: fit - Fit the observed counting rates to best model obtained by adjusting
;	the free parameters to F_MODEL.
;#     Change Option: sumfit,1 - Having fit it once, use those parameters in each fit as the defaults.
;#     Command:fit - Fit again.
;#                    
; Use         : .r run_near
;    
; Inputs      : None, this is a main program.
;               
; Opt. Inputs : None
;               
; Outputs     : None
;
; Opt. Outputs: None
;               
; Keywords    : 
;
; Calls	      :
;
; Common      : near_pin_resp (generally not needed)
;               
; Restrictions: 
;               
; Side effects: None.
;               
; Prev. Hist  : Based on command files created in 1996.
;
; Modified    : Version 1, richard.schwartz@gsfc.nasa.gov, 3-nov-1997.
;
;-            
;==============================================================================


common near_pin_resp, gain_mod, edges_out_sav, edges_in_sav, eloss_mat, newvalues
spex_proc,/cm,input='/'
;
; Make sure the site directory comes first
;
site = [path_dir('sdac/site'),path_dir('[sdac.site]')]
site = site( where(site ne '' ,nsite))
for i=0,nsite-1 do remove_path,site(i)
for i=0,nsite-1 do add_path,site(i)


;data,near,pin                                                                   
help,spex_current('p_drm_reader',inp='near_pin_resp')
help,spex_current('p_read_data',inp='rd_near_pin')
file = loc_file('960508_near2.txt',path=[curdir(),'~/soft/near','/umbra/schwartz','~schwartz'],count=fcount)
if fcount ne 1 then message,'Cannot find data file!!!!!!!!!'
break_file, file, disk, dir
command_list=['data,near,pin','f_model,f_con_lin','apar,.1,1.,1.,1.,1.,1.,7.5e-5,0.0','range_lo,,.2,,.4', $
	'directory,'+disk+dir, $
	'_1file,960508_near2.txt','graph','read_drm','back_order,0','back','t_h,c','zoom',$
	'sele','sp_xty,0','spyr,10,1e6','bspec','enrange','free,1,1,0,0,0,0,0,0','fit','sumfit,1','fit']

result = dialog_message(title='SPEX', 'Show helpful text for each step?',/ques)
if result eq 'Yes' then begin
	text = rd_ascii(loc_file('run_near.pro',path=[curdir(),path_dir('spex'),path_dir('near')]))
	line1= (where(strpos(text,';;;;;;;;; START DIALOG TEXT- DO NOT EDIT THIS LINE') eq 0))(0)+1
	text = text(line1:*)
	line2= (where(strpos(text,'; Use         :') eq 0))(0) -1
	text = text(0:line2)
	wstart = where(strpos(text,';#') eq 0, nstart)
	endif
for i=0,n_elements(command_list)-1 do begin
	if result eq 'Yes' then $
	rcheck = dialog_message(title='SPEX HELP',/info,strmid(text( wstart(i):wstart(i+1)-1),2,100))
	spex_proc,input=command_list(i),/cmode
	endfor
;
; Report the fitted values to the main program.
;
apar_arr = spex_current('apar_arr')
em_temp_near = apar_arr(0:1,*)
em_temp_near(1,*) = em_temp_near(1,*)*11.6
xselect  = spex_current('xselect') 
utbase   = getutbase()
xselect  = spex_current('xselect') + utbase
obsi     = spex_current('obsi')
backi     = spex_current('backi')
convi     = spex_current('convi')
timeback  = spex_current('tb')
tback  = timeback(*,0:1,0)

reduce_goes, utg, goes, sat=7, igoes=igoes7, itemp=itemp7, iem=iem7,$
timerange=minmax([tback(*),xselect(*)]), tback=tback, timesel=xselect
reduce_goes, utg, goes, sat=8, igoes=igoes8, itemp=itemp8, iem=iem8,$
timerange=minmax([tback(*),xselect(*)]), tback=tback, timesel=xselect
reduce_goes, utg, goes, sat=9, igoes=igoes9, itemp=itemp9, iem=iem9,$
timerange=minmax([tback(*),xselect(*)]), tback=tback, timesel=xselect

near_goes_fluxes = rebin(em_temp_near,2,n_elements(em_temp_near(0,*)),3)
for i=7,9 do begin
	goes_fluxes, near_goes_fluxes(1,*,i-7), near_goes_fluxes(0,*,i-7), flong, fshort, satellite = i, date=utbase
	near_goes_fluxes(0,0,i-7) = [transpose(flong),transpose(fshort)]
endfor
obs_goes_fluxes = [[[igoes7]],[[igoes8]],[[igoes9]]]
end               
