;+
; PROJECT:
;	SDAC
; NAME: 
;	spex_batse_preview	
; PURPOSE:
;	find file names and display energy range and aspect info
;	in preparation for file reading in SPEX of BATSE data.
; CALLING SEQUENCE:
;	spex_batse_preview, data_tipe, flare, det_id, in_files, in_directory, ;Inputs
;		      dir_path=dir_path, files, det_cosines, dsel, last_found, command              ;Outputs
; CATEGORY:
;	SPEX, BATSE, CGRO
; INPUTS:
;	data_tipe - instrument and data format, gets set to upper case
;	flare     - event number for BATSE
;	det_id    - detector id 
;	in_files  - 2 input file names to search for
;	in_directory - possible directory for data files
;	dir_queue - set of data directories to search including
;		    wildcards
;	standard_types - data formats integrated into SPEX environment	
;	source_radec - ra and dec of cosmic source, used if ra or dec non-zero
;	event_time - If flare isn't given for BATSE data, then event_time must be
; OUTPUTS:
;	files       - found filenames
;	det_cosines - detector aspects for event
;	dsel        - selected detectors (BATSE)
;	last_found  - structure with dataa_tipe, det_id, flare of found files
;	command     - string which directs output path in spex
;		      set to '' if no file found satisfying condition
;	text_out    - block of text reporting success of preview
; KEYWORDS:
;	BATSE_BURST - optional reference to BATSE database, if non-zero may override Flare
; OPTIONAL OUTPUTS:
;
; PROCEDURE: uses findfile to find standard file names in directories
;		given in dir_queue.  
;
; RESTRICTIONS: Defaults set for BATSE data, 14-apr-94. More to come.
; 
; MODIFICATION HISTORY: 
;	richard.schwartz@gsfc.nasa.gov, 29-sep-1997 from SPEX_PREVIEW
;	richard.schwartz@gsfc.nasa.gov, 2-oct-1997 files(0) set to '' on sher_only set. 
;-
;
;############################################################
;
pro spex_batse_preview, data_form, flare, det_id, in_files, in_directory_in, $
dir_queue=dir_queue, standard_types=standard_types, text_out=text, $
source_radec=source_radec, event_time=event_time, $
files, det_cosines, dsel, last_found, command, batse_burst=batse_burst
if strcompress(in_directory_in,/remove) eq '' then $
cd, current=in_directory $
else in_directory = in_directory_in


checkvar, files, strarr(2)+''
text = ''
;Locate the filenames consistent with the specified directory and filename
;save them in the string array, found_files. 


; Check to see if files can be found corresponding to data type and event
if strpos( data_form, 'DISCSP') ne -1 then data_form='DISCSP'
if strpos( data_form, 'CONT') ne -1 then data_form='CONT'
if strpos( data_form, 'DISCSP_CONT') ne -1 then data_form='DISCSP_CONT'

if strpos( data_form, 'STTE') ne -1 then data_form='STTE'
if strpos(data_form,'HER') ne -1 then $
if strpos(data_form,'SHER') NE -1 then data_form='SHERS' $
else data_form='HERS'

data_tipe = ['BATSE',data_form]

;	Check to see if data specifications have changed before trying to find the matching
;	file(s).
if strupcase(last_found.data(0)) ne strupcase(data_tipe(0)) or $
strupcase(last_found.data(1)) ne strupcase(data_tipe(1)) or $
last_found.flare ne flare or last_found.id ne det_id then dosearch=1 else dosearch=0

if dosearch then test_file = in_files

if data_form eq 'DISCSP_CONT' then begin
    merge_batse, flare_info=flare, filenames=filenames, det_ids=det_ids, error=error, $
    cos_spec=cos_spec, cos_lad=cos_lad, /find
    det_id=det_ids(0)
    det_cosines=cos_spec(det_id)
    
    endif

if total( abs(source_radec)) eq 0.0 then $	;only
det_cosines, flare=flare,cos_lad=cos_det,cos_spec=cos_spec, peak=peaktime $
else begin 
    ;   for now set cosines to 0
    cos_det=fltarr(8)
    cos_spec=fltarr(8)
    endelse
if (where(data_form eq ['CONT','HERS','DISCLA']))(0) ne -1 then $
det_cosines = cos_det else det_cosines=cos_spec
text=[text,'These are the detector cosines for '+data_tipe(0)+' '+data_tipe(1),$
string(/print,det_cosines)]
if data_form eq 'DISCSP' then begin
    low_edge = ((discsp_edges(fcheck(peaktime, event_time)))(0,*))(*)
    text = [text, 'DISCSP low edge (kev):', string(/print,low_edge)]
    endif
if (where(data_form eq ['CONT','DISCSP','DISCLA']))(0) ne -1 then begin
    if dosearch then begin
        batse_file_search, data_form, file_type='DATA', spex_test_dir = in_directory, $
        dir_queue=dir_queue, $
        test_file=test_file, det_id=det_id, flare=flare, out_fname=data_fname, out_dir=data_dir
        
        if data_fname(0) ne '' then files = concat_dir(data_dir(0), data_fname(0)) else begin
            text=[text,string(/print,'No file found satisfying data type requirements ',data_tipe),$
            string(/print,'For detector id ',det_id,' for flare ',flare)]
            goto, opcom
            endelse
        endif else text=[text,string(/print,'The data will be read from this file, ',files)]
    endif
if strpos(data_form,'HER') ne -1 or strpos(data_form,'STTE') ne -1 then begin
    sher_only = fcheck(spex_current('sher_only'),0)
    if sher_only then files(0)=''
    dsel = indgen(8) ;in case the sherb isn't used or available
    if not sher_only then begin
        if dosearch then begin
            batse_file_search, data_form, file_type='DATA', spex_test_dir = in_directory, $
            dir_queue=dir_queue, event_time=event_time, $
            test_file=test_file, det_id=det_id, flare=flare, $
            /sherb_only,  out_fname=data_fname, out_dir=data_dir, batse_burst=batse_burst
            if data_fname(0) ne '' then files = concat_dir(data_dir(0), data_fname(0)) else begin
                text = [text,string(/print, $
                'No (S)HERB file found satisfying data type requirements ',data_tipe),$
                string(/print,'For detector id ',det_id,' for flare ',flare)]
                files(0) =''
                ;			goto, opcom
                goto, findsher
                endelse
            endif
        
        if files(0) ne '' then begin
            read_sherb,files(0),0,/head,dsel=dsel,burst_time=btime
            low_edge = ((discsp_edges(btime))(1,dsel))(*)	
            text =[text,'These are the BATSE detectors stored in the SHERB and HERB files: ',$
            string(/print, dsel), $
            string(/print,'The lower edge for their pulse-height data is about ',low_edge), $
            string(/print,'The detector cosines for these detectors are: ',det_cosines(dsel))]
            endif
        endif
    
    ;Is the det_id one of the selected detectors (DSEL)?
    findsher:	checkvar, dsel, indgen(8)	;if no sherb, then all 8 are possible
    dosearch = dosearch or n_elements(files) ne 2
    if not dosearch then begin
        text = [text,'Found this (S)HER file, '+files(1)]
        endif else begin
        if (where( det_id eq dsel))(0) ne -1 or sher_only then begin
            found_sher = 0 ;assume sher not found
            ;Only looking for SHER data when there is SHERB data
            ;for that event, to be changed!!!
            data_dir = fcheck(data_dir, in_directory)
;stop
            batse_file_search, data_form, file_type='DATA', spex_test_dir = data_dir(0), $
            dir_queue=dir_queue, batse_burst=batse_burst, $
            test_file= test_file, det_id=det_id, flare=flare, $
            out_fname=sher_fname, out_dir=data_dir, /sher_only 
            
            ;Note!!! out_fname always returns the sher or her name in the second position!
            if sher_fname(1) ne '' then begin
                files = [files(0), concat_dir(data_dir(0), sher_fname(1))]
                found_sher=1
                endif 
            endif ;closes block to find sher
        if not fcheck(found_sher,0) then begin
            text = [text, string(/print,$
            'No (S)HER file found satisfying data type requirements ',data_tipe),$
            string(/print,'For detector id ',det_id,' for flare ',flare)]
            goto, opcom  ;escape
            endif else if files(0) eq ''  then begin
            read_flare, flare, flare_rec
            low_edge = ((discsp_edges(flare_rec.start_secs))(1,*))(*)	
            text =[text,'These are the BATSE detectors stored in the SHERB and HERB files: ',$
            string(/print, dsel), $
            string(/print,'The lower edge for their pulse-height data is about ',low_edge), $
            string(/print,'The detector cosines for these detectors are: ',det_cosines)]
            endif  ;closes sher_only test
        ;
        ; If we aren't looking at the Sun, then now it is time to figure out the 
        ; detector cosines
        ;
        if total( abs(source_radec)) ne 0.0 then begin
            case 1 of
                (where(strpos(strlowcase(files),'stte') ne -1))(0) ne -1:begin
                    read_stte, files((where(strpos(strlowcase(files),'stte') ne -1))(0) ), $
                    stte, dselb, det_id=det_id
                    bits, dselb, dsel
                    dsel = where( dsel )
                    ut_start = stte(0).ut
                    end
                (where(strpos(strlowcase(files),'herb') ne -1))(0) ne -1:begin
                    read_sherb, files((where(strpos(strlowcase(files),'sherb') ne -1))(0) ), $
                    det_id,0,dsel=dsel,burst_time=btime, /head
                    ut_start = btime
                    end
                (where(strpos(strlowcase(files),'her') ne -1))(0) ne -1:begin
                    read_sher, files((where(strpos(strlowcase(files),'her') ne -1))(0) ), $
                    data, ut_start
                    ut_start=avg(ut_start)
                    dsel = indgen(8)
                    end
                else:
                endcase
            ;
            ; Now that we have the time of the observation we can consult the pointing database
            ;
            gro_point, atime(ut_start), source_radec, cos_det, cos_spec
            if data_form eq 'HERS' then det_cosines=cos_det else det_cosines=cos_spec
            if data_form ne 'HERS' then low_edge = ((discsp_edges(ut_start))(1,*))(*)  else $
            low_edge = 	((discla_edges(ut_start))(0,*))(*)  
            text =[text,'These are the BATSE detectors available in the requested files: ',$
            string(/print, dsel), $
            string(/print,'The lower edge for their pulse-height data is about ',low_edge), $
            string(/print,'The detector cosines for these detectors are: ',det_cosines)]
            endif			;closes non-solar source test
        
        
        endelse ;closes dosearch block
    
    endif ;Close (S)HER block
return
opcom: command(0)='' ;return to command processor
end
