;+
; NAME:
;	SPEX_FLUX_TIME
;
; PURPOSE:
;	Returns a matrix of flux values, 1 for each energy and each set of model parameters
;	within the applicable energy range.
; CATEGORY:
;	SPEX, Modeling
;
; CALLING SEQUENCE:
;
;	SPEX_FLUX_TIME, Model, Energy, Dummy_pars, Ipar, Edges, Model_pars, Values
;
; INPUTS:
;	Model	: Name of model function, e.g. 'F_vth_bpow'
;	Energy	: Vector of first argument to Model function.  Usually a vector
;		energies in keV. Monotonically increasing.
;	Dummy_pars: Dummy parameters into which the variable Model_pars parameters are inserted
;	Ipar	: Insert the Model_pars into this position of Dummy_pars
;	Edges	: Energy edges of each channel.  Gives the range of applicability for
;		  Each Model_pars. 2 x #channels	
;	Model_pars:
;		  An array of model parameters, e.g. a set for each fit interval.
;		  2 x #edges(0,*) #intervals. 
;	
; OPTIONAL INPUTS:
;	
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;	Values	: Array of returns from Model, #Energy x #Intervals.
;
; OPTIONAL OUTPUTS:
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;
; CALLS:
; 	SPEX_MODEL_ARRAY
; RESTRICTIONS:
;
; PROCEDURE:
;	Loops over each set of parameters to save the user the grief of a visible loop.
;
; EXAMPLE:
;	
;   SPEX_FLUX_TIME, Model, Energy, Dummy_pars, Ipar, Edges, Model_pars, Values
;
; MODIFICATION HISTORY:
; 	Written by:	RAS, 23-March-1996
;	Version 1
;-
pro SPEX_FLUX_TIME, Model, Energy, Dummy_pars, Ipar, Edges, Model_pars, Values

spars = size(model_pars)
npar  = spars(1)	;number of varying parameters in Model_pars for replacement into dummy_pars
nints = spars(3)
nedg  = n_elements(edges(0,*))
apar = dummy_pars # (fltarr(1,nints)+1.)
if (size(energy))(0) eq 2 then edge_products, energy, mean=energym
values = fltarr( n_elements(energym), nints)

;Start at the lowest energy and work up
energy_low = 0.9 * energym(0)
for j=0,nedg-1 do begin
	if j lt nedg-1 then w = where( energym le edges(1,j) and energym gt energy_low, nw) else $
		w = where( energym gt energy_low, nw)
	energy_low = edges(1,j)
	if nw ge 1 then begin
		apar( ipar:ipar+npar-1,*) = model_pars(*,j,*)
		for i=0,nints-1 do values(w,i) = call_function( model, energym(w), apar(*,i) )
	endif
endfor

end

