;+
;
; NAME: 
;	SPEX_HANDLES
;
; PURPOSE:
;	Global data handling in SPEX.
;
; CATEGORY:
;	SPEX
;
; CALLING SEQUENCE:
;	SPEX_HANDLES, Search_str
;
; CALLS:
;	none
;
; INPUTS:
;	none
;
; OPTIONAL INPUTS:
;	Search_str -  A SPEX_HANDLES structure to look search for, data under this
;	handle is placed into SPEX_COMMONS.  If you want the newly written
;	Top_handle and Top_handle_id use '' for Search_str
;
; OUTPUTS:
;       Top_handle    - Returns the TOP_HANDLE structure when it finds it.
;	Top_handle_id - handle id of TOP_HANDLE variable
; OPTIONAL OUTPUTS:
;	none
;
; KEYWORDS:
;	TYPE	- 'INPUT', 'DRM', or 'FIT' indicating type of data to place into
;	handle storage
;	README  - a string with information about the data set, input or output.
;	SELECT_ID- a number to identify multiple data sets, as in fitting or
;	changed edges or drm's, this is a returned value
;	EVENT_ID - a string which uniquely identifies the initial dataset, '3030' or 
;	the elements in the array FILES	or the midpoint time of the dataset.
;	ERROR   - returns 1 if SEARCH_STR can't be found.
;
; COMMON BLOCKS:
;	none
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	Watch out.  Data storage could get huge.
;
; PROCEDURE:
;	This procedure causes the data values in spex_commons to be swapped between
;	the common area and the handle area.
;
; MODIFICATION HISTORY:
;	ras, 3 april 1996
;-
pro SPEX_HANDLES, Search_str, Top_handle, Top_handle_id, TYPE=TYPE, README= README, $
	SELECT_ID=SELECT_ID, EVENT_ID=EVENT_ID, ERROR=ERROR

error=0 

@spex_commons
;
; If there isn't a SEARCH_STR input, then data will be written into handle storage.
;
checkvar, search_str, ''
if datatype(search_str) eq 'STR' then $
if search_str(0) eq '' then begin
;
;Take the data in the common block and write it into the new handle
;
  tags_all='data_tipe, det_id, select_id, readme, event_id, curtime, edges, edg_units, uflux, ' + $
	'delta_light, count_2_flux, area, utbase,'+$
	'prm_spex, need'
  tags_in=tags_all+ ', cosine, ut, flux, eflux, ltime, rate, datamode, '+$
	'erate,  back, eback, wback, tb, def_tb, prm_spex, need'
  tags_drm=tags_all +' ,e_in, drm, eff'
  tags_fit=tags_all + ',xselect, iselect, wint, '+$
	' wuse, obsi, convi, eobsi, backi, ebacki, title,'+ $
 	'live, iegroup,  apar_arr, apar_last, apar_sigma, chi, batse_burst, utbase'

  type = strupcase(strtrim(fcheck(type,'INPUT'),2)) 
  case 1 of 
	type eq 'INPUT'   : tags= tags_in
	type eq 'DRM'     : tags= tags_drm
	type eq 'FIT'     : tags= tags_fit
	else: tags= tags_all
  endcase

  tags= str_sep(strcompress(tags,/rem),',')
  data_tipe=spex_current('data_tipe')
  utbase = getutbase()
  curtime = systime(1)-utime('1-jan-1988') ;systime is sec from 1-jan-1970
  checkvar, readme, ''
  if fcheck( event_id, '') eq '' then begin 
    files = spex_current('files') 	;data files are reasonably unique for id
    wgd = where( files ne '', ngd)
    case ngd of
	0: event_id = atime( avg(ut)+ utbase, /yohk) ;if no file used, then use time.
	1: event_id = files(wgd)
	else: event_id = arr2str( files, ',')
    endcase
  endif
;	Result = SPEX_HANDLES_STR( Data_tipe, Det_id, Type, Select_id, README=README, $
;	EVENT_ID=EVENT_ID)
  handle_str = spex_handles_str(data_tipe, det_id, type, fcheck( select_id, 0L), $
	event_id=event_id, readme=readme)
  top_ids = spex_handles_find( handle_str, top_handles=top_handles)
  if top_ids(0) gt 0 then handle_str.select_id = long( max( top_handles.select_id ) + 1)
;
; Now we are ready to create the new top handle, and load it with the identifying structure
;
  top_handle_id = handle_create()
  handle_value, /set, top_handle_id, handle_str
  handles(1) = handle_str 	; put the last top spex_handle saved into common
  top_handle = handle_str	; output argument
  select_id  = handle_str.select_id

  for i=0,n_elements(tags)-1 do begin
	result = execute( 'value = fcheck('+tags(i)+',0)' )
	result =handle_create( top_handle_id, value=value )
  endfor
  result = handle_create( top_handle_id,/first, value=tags )

endif else begin
;
; Use SEARCH_STR to find dataset of interest in read back into SPEX_COMMONS.
;
  hids = spex_handles_find( SEARCH_STR(0), top_handles=top_handles)
  if hids(0) eq 0 then begin
	printx,"ERROR, No dataset in Handles storage corresponding to SEARCH_STR."
	error=1
	help, search_str, /st
	return
  endif
;
; If select_id isn't set or doesn't match, then the last one is chosen
; 
  select_ids = top_handles.select_id 
  checkvar, select_id, max( select_ids )
  wsel = (where( select_id eq select_ids, nsel))(0)
  if nsel eq 0 then wsel = n_elements( select_ids ) -1
  hid = hids( wsel )
  top_handle = top_handle( wsel )
  readme = top_handle.readme
  event_id = top_handle.event_id
  select_id = top_handle.select_id
  type = top_handle.type
  ;
  ; Get the children of this handle.
  top_handle_id  = spex_handles_find( top_handle, cids, select_id=select_id )
  handle_value, cids(0),  tags
  for i=1,n_elements(tags)-1 do $
	result = execute( 'handle_value, cids(i), '+tags(i-1) )
  handles(0) = top_handle ;put the last spex_handle read into common
endelse
end


